package cn.estsh.i3plus.ext.mes.api.busi;

import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.ProdOrgModel;
import io.swagger.annotations.ApiOperation;

import java.util.List;

/**
 * @Description : 通用业务方法的调度接口
 * @Reference :
 * @Author : jack.jia
 * @CreateDate : 2019-04-11
 * @Modify:
 **/
public interface IProdExtOrgService {

    /**
     * 查询MES组织模型数据返回树结构
     *
     * @param organizeCode 组织代码
     * @return 组织模型数据返回树结构
     */
    @ApiOperation(value = "查询MES组织模型数据返回树结构")
    List queryMesDataTree(String organizeCode);

    /**
     * 新增mes组织模型信息(区域，生产线，工位)
     *
     * @param prodOrgModel 组织模型信息
     * @return 新增后的组织模型信息
     */
    Object insertMesProdOrg(ProdOrgModel prodOrgModel);

    /*************MES区域服务*****start***********/

    /**
     * 按条件分页查询区域信息
     *
     * @param mesArea 区域信息
     * @param pager   分页条件
     * @return 分页数据
     */
    @ApiOperation(value = "按条件分页查询区域信息")
    ListPager<MesArea> queryMesAreaByPager(MesArea mesArea, Pager pager);

    /**
     * 按条件查询区域所有数据
     *
     * @param mesArea 区域信息查询条件
     * @return 区域所有数据
     */
    @ApiOperation(value = "按条件查询区域所有数据")
    List<MesArea> queryMesAreaByProperties(MesArea mesArea);

    /**
     * 更新区域
     *
     * @param mesArea 区域
     * @return 更新后的区域
     */
    @ApiOperation(value = "更新区域", notes = "更新区域")
    MesArea updateMesArea(MesArea mesArea);

    /**
     * 更新区域状态
     *
     * @param id       区域id
     * @param status   更新状态
     * @param userName 操作人
     * @return 操作行数
     */
    @ApiOperation(value = "更新区域状态")
    int updateMesAreaStatusById(Long id, int status, String userName);

    /*************MES区域服务*****end***********/

    /*************MES生产线服务*****start***********/

    /**
     * 按条件分页查询生产线信息
     *
     * @param mesWorkCenter 产线查询条件
     * @param pager         分页条件
     * @return 分页数据
     */
    @ApiOperation(value = "按条件分页查询生产线信息")
    ListPager<MesWorkCenter> queryWorkCenterByPager(MesWorkCenter mesWorkCenter, Pager pager);

    /**
     * 按条件查询生产线所有数据
     *
     * @param workCenter 产线查询条件
     * @return 数据集合
     */
    @ApiOperation(value = "按条件查询生产线所有数据")
    List<MesWorkCenter> queryMesWorkCenterByProperties(MesWorkCenter workCenter);

    /**
     * 按org查询生产线所有数据
     *
     * @param org 组织代码
     * @return 生产线所有数据
     */
    @ApiOperation(value = "按org查询生产线所有数据")
    List<MesWorkCenter> queryMesWorkCenterByOrg(String org);

    /**
     * 更新生产线信息
     *
     * @param workCenter 产线
     * @return 更新后的产线信息
     */
    @ApiOperation(value = "更新生产线信息", notes = "更新生产线信息")
    MesWorkCenter updateMesWorkCenter(MesWorkCenter workCenter);

    /**
     * 更新生产线状态
     *
     * @param id       产线id
     * @param status   更新状态
     * @param userName 操作人
     * @return 操作行数
     */
    @ApiOperation(value = "更新生产线状态")
    int updateMesCenterStatusById(Long id, int status, String userName);

    /*************MES生产线服务*****end***********/

    /*************MES工位服务*****start***********/

    /**
     * 按条件分页查询工位信息
     *
     * @param mesWorkCell 工位查询条件
     * @param pager       分页条件
     * @return 分页数据
     */
    @ApiOperation(value = "按条件分页查询工位信息")
    ListPager<MesWorkCell> queryWorkCellByPager(MesWorkCell mesWorkCell, Pager pager);

    /**
     * 按条件查询工位所有数据
     *
     * @param workCell 工位查询条件
     * @return 数据集合
     */
    @ApiOperation(value = "按条件查询工位所有数据")
    List<MesWorkCell> queryMesWorkCellByProperties(MesWorkCell workCell);

    /**
     * 更新工位信息
     *
     * @param workCell 工位信息
     * @return 更新后的工位信息
     */
    @ApiOperation(value = "更新工位信息", notes = "更新工位信息")
    MesWorkCell updateMesWorkCell(MesWorkCell workCell);

    /**
     * 更新工位状态
     *
     * @param id       工位状态id
     * @param status   更新状态
     * @param userName 操作人
     * @return 操作行数
     */
    @ApiOperation(value = "更新工位状态")
    int updateMesCellStatusById(Long id, int status, String userName);

    /*************MES工位服务*****end***********/

    /*************MES工位参数服务*****start***********/

    /**
     * 按条件分页查询工位参数信息
     *
     * @param workCellParam 工位参数信息
     * @param pager         分页条件
     * @return 分页数据
     */
    @ApiOperation(value = "按条件分页查询工位参数信息")
    ListPager<MesWorkCellParam> queryMesWorkCellParamByPager(MesWorkCellParam workCellParam, Pager pager);

    /**
     * 新增工位参数配置信息
     *
     * @param workCellParam 工位参数信息
     * @return 新增后的工位参数信息
     */
    @ApiOperation(value = "新增工位参数信息", notes = "新增工位参数信息")
    MesWorkCellParam insertMesWorkCellParam(MesWorkCellParam workCellParam);

    /**
     * 更新工位参数配置信息
     *
     * @param workCellParam 更新工位参数信息
     * @return 更新后的更新工位参数信息
     */
    @ApiOperation(value = "更新工位参数信息", notes = "更新工位参数信息")
    MesWorkCellParam updateMesWorkCellParam(MesWorkCellParam workCellParam);

    /**
     * 更新工位参数状态(启用、禁用)
     *
     * @param id       工位参数状态id
     * @param status   更新状态
     * @param userName 操作人
     * @return 操作行数
     */
    @ApiOperation(value = "更新工位参数状态(启用、禁用)")
    int updateMesWorkCellParamStatusById(Long id, int status, String userName);

    /*************MES工位参数服务*****end***********/

    /*************MES工位参数配置服务*****start***********/

    /**
     * 按条件分页查询工位参数配置信息
     *
     * @param workCellParamCfg 工位参数配置信息
     * @param pager            分页条件
     * @return 分页数据
     */
    @ApiOperation(value = "按条件分页查询工位参数配置信息")
    ListPager<MesWorkCellParamCfg> queryMesWorkCellParamCfgByPager(MesWorkCellParamCfg workCellParamCfg, Pager pager);

    /**
     * 新增工位参数配置信息
     *
     * @param workCellParamCfg 工位参数配置信息
     * @return 新增后的工位参数配置信息
     */
    @ApiOperation(value = "新增工位参数配置信息", notes = "新增工位参数配置信息")
    MesWorkCellParamCfg insertMesWorkCellParamCfg(MesWorkCellParamCfg workCellParamCfg);

    /**
     * 更新工位参数配置信息
     *
     * @param workCellParamCfg 工位参数配置信息
     * @return 更新后的工位参数配置信息
     */
    @ApiOperation(value = "更新工位参数配置信息", notes = "更新工位参数配置信息")
    MesWorkCellParamCfg updateMesWorkCellParamCfg(MesWorkCellParamCfg workCellParamCfg);

    /**
     * 更新工位参数配置状态(启用、禁用)
     *
     * @param id       工位参数配置信息id
     * @param status   更新状态
     * @param userName 操作人
     * @return 操作行数
     */
    @ApiOperation(value = "更新工位参数配置状态(启用、禁用)")
    int updateMesWorkCellParamCfgStatusById(Long id, int status, String userName);

    /*************MES工位参数配置服务*****end***********/

}
