package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesBomService;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesBom;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * bom信息
 */
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesBom")
public class MesBomController extends BaseMesController<MesBom> {

    @Autowired
    private IMesBomService bomService;

    @GetMapping(value = "/find-bom-version")
    @ApiOperation(value = "查询生产版本信息")
    public ResultBean findBomVersionByPartNo(MesBom bom) {
        try {
            ValidatorBean.checkNotNull(bom.getPartNo(), "父零件号不能为空");
            bom.setOrganizeCode(AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(bomService.findBomVersionByPartNo(bom));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/find-bom-code")
    @ApiOperation(value = "根据生产版本查询BomCode")
    public ResultBean findBomCodeByPartNoAndBomVersion(MesBom bom) {
        try {
            ValidatorBean.checkNotNull(bom.getPartNo(), "父零件号不能为空");
            ValidatorBean.checkNotNull(bom.getBomVersion(), "BOM版本不能为空");
            bom.setOrganizeCode(AuthUtil.getOrganizeCode());
            return ResultBean.success("查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(bomService.findBomCodeByPartNoAndBomVersion(bom));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @GetMapping(value = "/tree/query-first-floor")
    @ApiOperation(value = "查询MES组织模型数据返回树结构-只获取一层")
    public ResultBean queryMesDataTreeByFirstFloor(MesBom bom) {
        try {
            bom.setOrganizeCode(AuthUtil.getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setResultList(bomService.queryMesDataTreeByFirstFloor(bom));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @GetMapping(value = "/tree/query")
    @ApiOperation(value = "查询MES组织模型数据返回树结构-获取所有")
    public ResultBean queryMesDataTree(MesBom bom, Pager pager) {
        try {
            ValidatorBean.checkNotNull(bom.getPartNo(), "父零件号不能为空");
            ValidatorBean.checkNotNull(bom.getBomVersion(), "BOM版本不能为空");
            ValidatorBean.checkNotNull(bom.getBomCode(), "BOM代码不能为空");
            ValidatorBean.checkNotNull(bom.getEffStartTime(), "生效时间不能为空");
            bom.setOrganizeCode(AuthUtil.getOrganizeCode());
            return ResultBean.success("操作成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode())
                    .setListPager(bomService.queryMesDataTree(bom, pager));
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }
}
