package cn.estsh.i3plus.ext.mes.apiservice.controller.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderToSapService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.excel.MesExcelTool;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.enumutil.ResourceEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesConfig;
import cn.estsh.i3plus.pojo.mes.bean.MesProductVersion;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.model.MesWorkOrderModel;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.i3plus.pojo.model.wms.WmsProductModel;
import cn.estsh.impp.framework.boot.auth.AuthUtil;
import cn.estsh.impp.framework.boot.exception.ImppBusiException;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.RedisCacheTool;
import cn.estsh.impp.framework.boot.util.ResultBean;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

@Api(description = "工单主页面")
@RestController
@RequestMapping(MesCommonConstant.MES_YANFEN + "/mesWorkOrder")
public class MesWorkOrderController extends BaseMesController<MesWorkOrder> {

    @Autowired
    private IMesWorkOrderService mesWorkOrderService;

    @Autowired
    private IMesWorkOrderToSapService mesWorkOrderToSapService;

    @Autowired
    private IMesConfigService mesConfigService;

    @ApiOperation(value = "查询生产版本", notes = "查询生产版本")
    @GetMapping("/mesProductVersion/query")
    public ResultBean queryMesProdRuleByPager(MesProductVersion mesProductVersion, Pager pager){
        try {
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            mesProductVersion.setOrganizeCode(organizeCode);
            ListPager<MesProductVersion> productVersionListPager = mesWorkOrderService.queryMesProductVersionByPager(mesProductVersion, pager);
            return ResultBean.success("生产版本查询成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode()).
                    setListPager(productVersionListPager);
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/import-mes-work-order")
    @ApiOperation(value = "导入数据")
    public ResultBean importMesWorkOrderExcel(@RequestParam("file") MultipartFile file, @RequestParam("status") Integer type) {
        try {
            MesExcelTool excelTool = new MesExcelTool(entityManager, RedisCacheTool.getImppRedis());
            List<MesWorkOrderModel> beanList = excelTool.importData(file.getOriginalFilename(), file.getInputStream(), MesWorkOrderModel.class);

            if(CollectionUtils.isEmpty(beanList)){
                MesException.throwMesBusiException("导入数据不存在");
            }
            String userName  = AuthUtil.getSessionUser().getUserName();
            String organizeCode = AuthUtil.getOrganize().getOrganizeCode();
            List<MesWorkOrder> mesWorkOrderList = new ArrayList<>();
            // 导入数据初始化
            for (MesWorkOrderModel bean : beanList) {
                MesWorkOrder mesWorkOrder = new MesWorkOrder();
                BeanUtils.copyProperties(bean, mesWorkOrder);
                mesWorkOrder.setWorkOrderType(type);
                ConvertBean.serviceModelInitialize(mesWorkOrder, userName);
                mesWorkOrder.setOrganizeCode(organizeCode);
                mesWorkOrderList.add(mesWorkOrder);
            }
            mesWorkOrderService.saveMesWorkOrderExcel(mesWorkOrderList);

            return ResultBean.success("导入成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/doProductReport")
    @ApiOperation(value = "生产报工")
    public ResultBean doProductReport(@RequestBody MesWorkOrder mesWorkOrder) {
        try {
            //属性校验
            if(Objects.isNull(mesWorkOrder.getNum()) || mesWorkOrder.getNum() <= 0){
                MesException.throwMesBusiException("数量不能小于等于0");
            }
            //新增初始化
            mesWorkOrder.setOrganizeCode(AuthUtil.getOrganizeCode());
            ConvertBean.serviceModelInitialize(mesWorkOrder, AuthUtil.getSessionUser().getUserName());
            mesWorkOrderService.doProductReport(mesWorkOrder);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }

    @PostMapping(value = "/doProductReportExt")
    @ApiOperation(value = "生产报工(武汉)")
    public ResultBean doProductReportExt(@RequestBody MesWorkOrder mesWorkOrder) {
        try {
            ValidatorBean.checkNotNull(mesWorkOrder.getId(),"工单id不能为空");
            ValidatorBean.checkNotNull(mesWorkOrder.getNum(),"报工数量不能为空");
            ValidatorBean.checkNotZero(mesWorkOrder.getNum(),"报工数量不能为0");
            if(Objects.isNull(mesWorkOrder.getNum()) || mesWorkOrder.getNum() <= 0){
                MesException.throwMesBusiException("数量不能小于等于0");
            }
            mesWorkOrder.setOrganizeCode(AuthUtilExt.getOrganizeCode());
            ConvertBean.serviceModelUpdate(mesWorkOrder, AuthUtilExt.getUserName());
            //新增初始化
            mesWorkOrderService.doProductReportExt(mesWorkOrder);
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }


    @GetMapping(value = "/sendToSap")
    @ApiOperation(value = "发送给sap")
    public ResultBean sendToSap(String organizeCode) {
        try {
            //查询汇总工单数量
            List<MesWorkOrderModel> mesWorkOrderList = mesWorkOrderService.sumMesWorkOrderQtyToSap(organizeCode);
            if (!CollectionUtils.isEmpty(mesWorkOrderList)){
                mesWorkOrderToSapService.doMesWorkOrder(organizeCode,"",mesWorkOrderList);
            }
            return ResultBean.success("操作成功")
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException e) {
            return ResultBean.fail(e).build();
        } catch (Exception e) {
            return ResultBean.fail(e);
        }
    }

    @PostMapping(value = "/report-work-reversal")
    @ApiOperation(value = "反向冲销")
    public ResultBean doReportWorkReversal(@RequestBody WmsProductModel model) {
        try {
            ValidatorBean.checkNotNull(model.getOrganizeCode(),"工厂代码不能为空");
            ValidatorBean.checkNotNull(model.getQty(),"数量不能为空");
            ValidatorBean.checkNotNull(model.getPartNo(),"零件号不能为空");

            mesWorkOrderService.doReportWorkReversal(model);
            return ResultBean.success("反向冲销成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/save-work-print-queue")
    @ApiOperation(value = "新增打印工单打印队列")
    public ResultBean doSaveWorkOrderPrintQueue(@RequestBody MesWorkOrder workOrder) {
        try {

            ValidatorBean.checkNotNull(workOrder.getOrganizeCode(),"工厂代码不能为空");

            mesWorkOrderService.doSaveWorkOrderPrintQueue(workOrder);
            return ResultBean.success("工单打印队列新增成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }

    @PostMapping(value = "/work-order-reprint")
    @ApiOperation(value = "装配目视单补打")
    public ResultBean doReprintWorkOrder(Long[] ids) {
        try {

            ValidatorBean.checkNotNull(ids,"工单不能为空");

            return ResultBean.success("装配目视单补打成功")
                    .setResultObject(mesWorkOrderService.doReprintWorkOrder(ids, AuthUtil.getOrganize().getOrganizeCode(), AuthUtil.getSessionUser().getUserName()))
                    .setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


    @PutMapping(value = "/reopen")
    @ApiOperation(value = "重新打开")
    public ResultBean doReopen(Long[] ids) {
        try {
            ValidatorBean.checkNotNull(ids);
            mesWorkOrderService.doReopen(ids, AuthUtil.getSessionUser().getUserName(),AuthUtil.getOrganizeCode());
            return ResultBean.success("重新打开成功").setCode(ResourceEnumUtil.MESSAGE.SUCCESS.getCode());
        } catch (ImppBusiException busExcep) {
            return ResultBean.fail(busExcep);
        } catch (Exception e) {
            return ImppExceptionBuilder.newInstance().buildExceptionResult(e);
        }
    }


}
