package cn.estsh.i3plus.ext.mes.apiservice.daoimpl;

import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesShippingOrderManagementDetailDao;
import cn.estsh.i3plus.ext.mes.pojo.model.MesShippingOrderManagementDetailModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderPart;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagement;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesShippingOrderManagementDetailDaoImpl implements IMesShippingOrderManagementDetailDao {


    @Autowired
    private EntityManager entityManager;

    @Override
    public int queryOrderDetailGroupByPartCount(MesShippingOrderManagementDetail detail) {

        StringBuffer hql = new StringBuffer("select count(1)  ");
        hql.append(" from mes_shipping_detail as msd");
        hql.append(" where msd.organize_code = :organizeCode ");
        hql.append(" and msd.is_deleted = :isDeleted ");
        hql.append(" and msd.is_valid = :isValid ");

        hql.append(" and msd.pid = :pid ");

        hql.append(" group by msd.part_no ,msd.part_name,msd.cust_part_no,msd.unit ");

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("pid", detail.getPid());

        List list = query.getResultList();

        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }

        return list.size();
    }

    @Override
    public List<MesShippingOrderManagementDetailModel> queryOrderDetailGroupByPartInfo(MesShippingOrderManagementDetail detail, Pager pager) {


        StringBuffer hql = new StringBuffer("select msd.part_no ,msd.part_name, msd.cust_part_no,sum( msd.plan_qty) as planQty , sum( msd.actual_qty) as actualQty,  msd.unit ");
        hql.append(" from mes_shipping_detail as msd");
        hql.append(" where msd.organize_code = :organizeCode ");
        hql.append(" and msd.is_deleted = :isDeleted ");
        hql.append(" and msd.is_valid = :isValid ");
        hql.append(" and msd.pid = :pid ");

        hql.append(" group by msd.part_no ,msd.part_name,msd.cust_part_no,msd.unit ");

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("pid", detail.getPid());


        query.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());

        List list = query.getResultList();
        List<MesShippingOrderManagementDetailModel> modelList = new ArrayList<>();

        for (Object result : list) {
            Object[] cells = (Object[]) result;
            MesShippingOrderManagementDetailModel model = new MesShippingOrderManagementDetailModel();
            model.setPartNo(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setPartName(StringUtils.isEmpty(cells[1]) ? "" : String.valueOf(cells[1]));
            model.setCustPartNo(StringUtils.isEmpty(cells[2]) ? "" : String.valueOf(cells[2]));
            model.setActualQty(StringUtils.isEmpty(cells[3]) ? 0 : Integer.parseInt(String.valueOf(cells[3])));
            model.setActualQty(StringUtils.isEmpty(cells[4]) ? 0 : Integer.parseInt(String.valueOf(cells[4])));
            model.setUnit(StringUtils.isEmpty(cells[5]) ? "" : String.valueOf(cells[5]));
            modelList.add(model);
        }

        return modelList;
    }

    @Override
    public ListPager<MesShippingOrderManagementDetailModel> queryMesShippingDetailsByPager(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel, Pager pager) {
        StringBuilder dataHql = new StringBuilder();
        dataHql.append(" select new ").append(MesShippingOrderManagementDetailModel.class.getName()).append("(m,d)");
        //共用HQL
        String commonHql = getCommonHql(mesShippingOrderManagementDetailModel);
        //行数
        Query countQuery = entityManager.createQuery("select count(1) " + commonHql);
        setParameter(mesShippingOrderManagementDetailModel, countQuery);
        //数据
        Query dataQuery = entityManager.createQuery(dataHql.append(commonHql).append(getOrderBy(mesShippingOrderManagementDetailModel)).toString(), MesShippingOrderManagementDetailModel.class);
        setParameter(mesShippingOrderManagementDetailModel, dataQuery);
        if (Objects.isNull(pager)) return new ListPager<>(dataQuery.getResultList(), pager);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        if (count <= 0) return new ListPager<>(new ArrayList<>(), pager);
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());
        return new ListPager<>(dataQuery.getResultList(), pager);
    }

    private void setParameter(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel, Query query) {
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("organizeCode", mesShippingOrderManagementDetailModel.getOrganizeCode());
        query.setParameter("status", MesExtEnumUtil.SHIPPING_ORDER_DETAIL_SHIPPING_STATUS.SHIPPINGED.getValue());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getShippingType()))
            query.setParameter("shippingType", mesShippingOrderManagementDetailModel.getShippingType());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getStartScanTimeStart()))
            query.setParameter("startScanTimeStart", mesShippingOrderManagementDetailModel.getStartScanTimeStart());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getStartScanTimeEnd()))
            query.setParameter("startScanTimeEnd", mesShippingOrderManagementDetailModel.getStartScanTimeEnd());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getEndScanTimeStart()))
            query.setParameter("endScanTimeStart", mesShippingOrderManagementDetailModel.getEndScanTimeStart());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getEndScanTimeEnd()))
            query.setParameter("endScanTimeEnd", mesShippingOrderManagementDetailModel.getEndScanTimeEnd());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getPublishTimeStart()))
            query.setParameter("publishTimeStart", mesShippingOrderManagementDetailModel.getPublishTimeStart());
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getPublishTimeEnd()))
            query.setParameter("publishTimeEnd", mesShippingOrderManagementDetailModel.getPublishTimeEnd());
    }

    private String getOrderBy(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel) {
        StringBuilder orderBy = new StringBuilder();
        if (StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getOrderByParam()) || StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getAscOrDesc())) {
            mesShippingOrderManagementDetailModel.setOrderByParam("createDatetime");
            mesShippingOrderManagementDetailModel.setAscOrDesc(CommonEnumUtil.ASC_OR_DESC.DESC.getValue());
        }
        return orderBy.append(" order by d.").append(mesShippingOrderManagementDetailModel.getOrderByParam()).append(" ").append(CommonEnumUtil.ASC_OR_DESC.DESC.getValue() == mesShippingOrderManagementDetailModel.getAscOrDesc() ? "desc" : "asc").toString();
    }

    private String getCommonHql(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel) {
        StringBuilder commonHql = new StringBuilder();
        commonHql.append(" from MesShippingOrderManagement m left join MesShippingOrderManagementDetail d on m.id = d.pid");
        commonHql.append(" where m.isDeleted=:isDeleted and m.isValid = :isValid and m.organizeCode = :organizeCode ");
        commonHql.append(" and d.isDeleted=:isDeleted and d.isValid = :isValid and d.organizeCode = :organizeCode ");
        commonHql.append(" and d.status = :status ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getShippingCode()))
            commonHql.append(" and m.shippingCode like '%").append(mesShippingOrderManagementDetailModel.getShippingCode()).append("%' ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getBarcode()))
            commonHql.append(" and d.barcode like '%").append(mesShippingOrderManagementDetailModel.getBarcode()).append("%' ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getPartNo()))
            commonHql.append(" and d.partNo like '%").append(mesShippingOrderManagementDetailModel.getPartNo()).append("%' ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getCustPartNo()))
            commonHql.append(" and d.custPartNo like '%").append(mesShippingOrderManagementDetailModel.getCustPartNo()).append("%' ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getShippingType()))
            commonHql.append(" and m.shippingType = :shippingType ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getCustCode()))
            commonHql.append(" and m.custCode like '%").append(mesShippingOrderManagementDetailModel.getCustCode()).append("%' ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getStartScanTimeStart()))
            commonHql.append(" and m.startScanTime >= :startScanTimeStart ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getStartScanTimeEnd()))
            commonHql.append(" and m.startScanTime <= :startScanTimeEnd ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getEndScanTimeStart()))
            commonHql.append(" and m.endScanTime >= :endScanTimeStart ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getEndScanTimeEnd()))
            commonHql.append(" and m.endScanTime <= :endScanTimeEnd ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getPublishTimeStart()))
            commonHql.append(" and m.publishTime >= :publishTimeStart ");
        if (!StringUtils.isEmpty(mesShippingOrderManagementDetailModel.getPublishTimeEnd()))
            commonHql.append(" and m.publishTime <= :publishTimeEnd ");
        return commonHql.toString();
    }
    @Override
    public List<MesShippingOrderManagementDetail> queryMesShippingOrderManagementDetail(MesWorkOrderPart detail ) {
        StringBuffer hql = new StringBuffer("" +
                " SELECT " +
                " SUBSTRING_INDEX(sd.bar_code, ',', 1) AS  bar_code, " +
                " sd.plan_qty, " +
                " sd.actual_qty, " +
                " s.shipping_code, " +
                " sd.part_no  " +
                " from  mes_shipping_detail  sd " +
                " left join mes_shipping s on s.id=sd.pid  " +
                " WHERE  sd.status in (20) " +//todo 发运 20
                " and sd.bar_code != '' and bar_code is not NULL ");
        hql.append(" and sd.organize_code = :organizeCode ");
        hql.append(" and sd.is_deleted = :isDeleted ");
        hql.append(" and sd.is_valid = :isValid ");

        if (!StringUtil.isEmpty(detail.getShippingCode())){
            hql.append(" and  s.shipping_code = '"+detail.getShippingCode()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getTotalPartNo())){
            hql.append(" and  sd.part_no = '"+detail.getTotalPartNo()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getWorkOrderNo())){
            hql.append(" and  sd.bar_code like '"+detail.getWorkOrderNo()+"%' ");
        }

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());

        List list = query.getResultList();
        List<MesShippingOrderManagementDetail> modelList = new ArrayList<>();

        for (Object result : list) {
            Object[] cells = (Object[]) result;
            MesShippingOrderManagementDetail model = new MesShippingOrderManagementDetail();
            model.setBarcode(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setPlanQty(StringUtils.isEmpty(cells[1]) ? 1 : Integer.parseInt(String.valueOf(cells[1])));
            model.setActualQty(StringUtils.isEmpty(cells[2]) ? 1 : Integer.parseInt(String.valueOf(cells[2])));
            model.setShippingCode(StringUtils.isEmpty(cells[3]) ? "" : String.valueOf(cells[3]));
            model.setPartNo(StringUtils.isEmpty(cells[4]) ? "" : String.valueOf(cells[4]));
            modelList.add(model);
        }

        return modelList;
    }
    @Override
    public int queryDetailGroupByPartNo(MesWorkOrderPart detail) {

        StringBuffer hql = new StringBuffer("select count(1)" +
                " from mes_work_order_part  wop  " +
                " left join mes_work_order wo on wop.work_order_no=wo.work_order_no " );
        hql.append(" WHERE wop.organize_code = :organizeCode ");
        hql.append(" and wop.is_deleted = :isDeleted ");
        hql.append(" and wop.is_valid = :isValid ");
        if (!StringUtil.isEmpty(detail.getModifyDateTimeStart())){
            hql.append(" and  wo.shipping_time >= '"+detail.getModifyDateTimeStart()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getModifyDateTimeEnd())){
            hql.append(" and  wo.shipping_time <= '"+detail.getModifyDateTimeEnd()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getPartNo())){
            hql.append(" and wop.part_no ='"+detail.getPartNo()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getWorkCenterCode())){
            hql.append(" and wop.work_center_code ='"+detail.getWorkCenterCode()+"' ");
        }
        hql.append(" and wop.work_order_no in (:codeList) ");

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("codeList", detail.getShippingDetails().stream().map(MesShippingOrderManagementDetail::getBarcode).collect(Collectors.toList()));


        List list = query.getResultList();

        if (CollectionUtils.isEmpty(list)) {
            return 0;
        }

        return list.size();
    }
    @Override
    public List<MesWorkOrderPart> queryOrderDetailGroupByPartInfo(MesWorkOrderPart detail, Pager pager) {

        StringBuffer hql = new StringBuffer("" +
                "SELECT   " +
                " wo.work_center_code, " +
                " wop.part_no, " +
                " wop.part_name, " +
                " wop.item_qty, " +
                " wo.shipping_time " +
                " from mes_work_order_part  wop  " +
                " left join mes_work_order wo on wop.work_order_no=wo.work_order_no " );
        hql.append(" WHERE wop.organize_code = :organizeCode ");
        hql.append(" and wop.is_deleted = :isDeleted ");
        hql.append(" and wop.is_valid = :isValid ");
        if (!StringUtil.isEmpty(detail.getModifyDateTimeStart())){
            hql.append(" and  wo.shipping_time >= '"+detail.getModifyDateTimeStart()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getModifyDateTimeEnd())){
            hql.append(" and  wo.shipping_time <= '"+detail.getModifyDateTimeEnd()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getPartNo())){
            hql.append(" and wop.part_no ='"+detail.getPartNo()+"' ");
        }
        if (!StringUtil.isEmpty(detail.getWorkCenterCode())){
            hql.append(" and wop.work_center_code ='"+detail.getWorkCenterCode()+"' ");
        }
        hql.append(" and wop.work_order_no in (:codeList) ");

        Query query = entityManager.createNativeQuery(hql.toString());
        query.setParameter("organizeCode", detail.getOrganizeCode());
        query.setParameter("isValid", CommonEnumUtil.TRUE_OR_FALSE.TRUE.getValue());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("codeList", detail.getShippingDetails().stream().map(MesShippingOrderManagementDetail::getBarcode).collect(Collectors.toList()));


        query.setFirstResult(pager.getStartRow()).setMaxResults(pager.getPageSize());

        List list = query.getResultList();
        List<MesWorkOrderPart> modelList = new ArrayList<>();

        for (Object result : list) {
            Object[] cells = (Object[]) result;
            MesWorkOrderPart model = new MesWorkOrderPart();
            model.setWorkCenterCode(StringUtils.isEmpty(cells[0]) ? "" : String.valueOf(cells[0]));
            model.setPartNo(StringUtils.isEmpty(cells[1]) ? "" : String.valueOf(cells[1]));
            model.setPartName(StringUtils.isEmpty(cells[2]) ? "" : String.valueOf(cells[2]));
            model.setItemQty(StringUtils.isEmpty(cells[3]) ? 1 : Double.parseDouble(String.valueOf(cells[3])));
            model.setShippingTime(StringUtils.isEmpty(cells[4]) ? "" : String.valueOf(cells[4]));
            modelList.add(model);
        }

        return modelList;
    }
}
