package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IMesYfReportDao;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportConditionModel;
import cn.estsh.i3plus.ext.mes.pojo.model.report.MesDeviceDataQueryReportResultModel;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 报表-数据方法实现
 * @Reference :
 * @Author : logic
 * @CreateDate : 2024/6/15 13:30
 * @Modify:
 **/
@Service
public class MesYfReportDaoImpl implements IMesYfReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<MesDeviceDataQueryReportResultModel> queryMesDeviceDataByPager(MesDeviceDataQueryReportConditionModel queryReportModel, Pager pager) {
        StringBuilder dataQuerySql = new StringBuilder();
        StringBuilder countQuerySql = new StringBuilder();
        StringBuilder commonQuerySql = new StringBuilder();
        dataQuerySql.append("select new ")
                .append(MesDeviceDataQueryReportResultModel.class.getName())
                .append("(organizeCode, workCenterCode, workCellCode, workOrderNo, serialNumber, ")
                .append("logType, message, equipmentCode, equipmentName, createDatetime, createUser, " +
                        "modifyDatetime, modifyUser) ");
        countQuerySql.append("select count(1) ");
        commonQuerySql.append("from MesWorkCellScanMonitorLog " +
                "where isValid = :isValid and isDeleted = :isDeleted and organizeCode = :organizeCode " +
                "and createDatetime >= :createDateTimeStart and createDatetime <= :createDateTimeEnd ");
        //非空查询条件拼接语句
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            commonQuerySql.append("and workCenterCode = :workCenterCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            commonQuerySql.append("and workCellCode = :workCellCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            commonQuerySql.append("and workOrderNo = :workOrderNo ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            commonQuerySql.append("and serialNumber = :serialNumber ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            commonQuerySql.append("and equipmentCode = :equipmentCode ");
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            commonQuerySql.append("and logType = :logType ");
        }
        Query dataQuery = entityManager.createQuery(dataQuerySql.append(commonQuerySql).toString(), MesDeviceDataQueryReportResultModel.class);
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        dataQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        dataQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        Query countQuery = entityManager.createQuery(countQuerySql.append(commonQuerySql).toString());
        countQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        countQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        countQuery.setParameter("organizeCode", queryReportModel.getOrganizeCode());
        countQuery.setParameter("createDateTimeStart", queryReportModel.getCreateDateTimeStart());
        countQuery.setParameter("createDateTimeEnd", queryReportModel.getCreateDateTimeEnd());
        if (!StringUtils.isEmpty(queryReportModel.getWorkCenterCode())) {
            dataQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
            countQuery.setParameter("workCenterCode", queryReportModel.getWorkCenterCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkCellCode())) {
            dataQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
            countQuery.setParameter("workCellCode", queryReportModel.getWorkCellCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getWorkOrderNo())) {
            dataQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
            countQuery.setParameter("workOrderNo", queryReportModel.getWorkOrderNo());
        }
        if (!StringUtils.isEmpty(queryReportModel.getSerialNumber())) {
            dataQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
            countQuery.setParameter("serialNumber", queryReportModel.getSerialNumber());
        }
        if (!StringUtils.isEmpty(queryReportModel.getEquipmentCode())) {
            dataQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
            countQuery.setParameter("equipmentCode", queryReportModel.getEquipmentCode());
        }
        if (!StringUtils.isEmpty(queryReportModel.getLogType())) {
            dataQuery.setParameter("logType", queryReportModel.getLogType());
            countQuery.setParameter("logType", queryReportModel.getLogType());
        }
        //本次查询满足条件的总数
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        //查询本页数据
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<MesDeviceDataQueryReportResultModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

}
