package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.IShipmentDetailReportDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.report.ShippingReportModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 发运明细报表 Dao实现
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/29 9:53
 * @Modify:
 **/
@Service
public class ShipmentDetailReportDaoImpl implements IShipmentDetailReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<ShippingReportModel> queryReport(ShippingReportModel model, Pager pager) {
        StringBuilder dataQueryHql = new StringBuilder();
        StringBuilder countQueryHql = new StringBuilder();
        StringBuilder commonQueryHql = new StringBuilder();

        dataQueryHql.append("select new ").append(ShippingReportModel.class.getName()).append(" ( ");

        // 添加所有需要查询的字段
        dataQueryHql.append("momd.createDatetime AS createDate,");
        dataQueryHql.append("momd.createUser AS createUser,");
        dataQueryHql.append("momd.isValid AS isValid,");
        dataQueryHql.append("momd.isDeleted AS isDeleted,");
        dataQueryHql.append("momd.modifyDatetime AS modifyDate,");
        dataQueryHql.append("momd.modifyUser AS modifyUser,");
        dataQueryHql.append("momd.organizeCode AS organizeCode,");
        dataQueryHql.append("momd.actualQty AS actualQty,");
        dataQueryHql.append("momd.custInfoSeq AS custInfoSeq,");
        dataQueryHql.append("momd.custOrderNo AS custOrderNo,");
        dataQueryHql.append("momd.custPartNo AS custPartNo,");
        dataQueryHql.append("momd.logicSeq AS logicSeq,");
        dataQueryHql.append("momd.paidQty AS paidQty,");
        dataQueryHql.append("momd.partName AS partName,");
        dataQueryHql.append("momd.partNo AS partNo,");
        dataQueryHql.append("momd.planQty AS planQty,");
        dataQueryHql.append("momd.status AS dStatus,");
        dataQueryHql.append("momd.supplierCode AS supplierCode,");
        dataQueryHql.append("momd.unit AS dUnit,");
        dataQueryHql.append("momd.barcode AS barCode,");
        dataQueryHql.append("momd.vin AS vin,");
        dataQueryHql.append("momd.assyCode AS assyCode,");
        dataQueryHql.append("momd.carSerialCode AS carSerialCode,");

        dataQueryHql.append("ms.checkSeqCode AS checkSeqCode,");
        dataQueryHql.append("ms.custCode AS custCode,");
        dataQueryHql.append("ms.custDockCode AS custDockCode,");
        dataQueryHql.append("ms.custOrganizeCode AS custOrganizeCode,");
        dataQueryHql.append("ms.custName AS custName,");
        dataQueryHql.append("ms.custOrganizeName AS custOrganizeName,");
        dataQueryHql.append("ms.dateSource AS dateSource,");
        dataQueryHql.append("ms.endScanTime AS endScanTime,");
        dataQueryHql.append("ms.lastPrintTime AS lastPrintTime,");
        dataQueryHql.append("ms.lastPrintUser AS lastPrintUser,");
        dataQueryHql.append("ms.printCount AS printCount,");
        dataQueryHql.append("ms.publishFlag AS publishFlag,");
        dataQueryHql.append("ms.publishTime AS publishTime,");
        dataQueryHql.append("ms.rackNo AS rackNo,");
        dataQueryHql.append("ms.roadProcess AS roadProcess,");
        dataQueryHql.append("ms.sendDockCode AS sendDockCode,");
        dataQueryHql.append("ms.seq AS seq,");
        dataQueryHql.append("ms.shippingGroupName AS shippingGroupName,");
        dataQueryHql.append("ms.shippingCode AS shippingCode,");
        dataQueryHql.append("ms.shippingType AS shippingType,");
        dataQueryHql.append("ms.startScanTime AS startScanTime,");
        dataQueryHql.append("ms.status AS status,");
        dataQueryHql.append("ms.shippingGroupCode AS shippingGroupCode,");
        dataQueryHql.append("ms.orderCode AS orderCode,");
        dataQueryHql.append("ms.scanSeqWay AS scanSeqWay");
        dataQueryHql.append(")");

        countQueryHql.append("select count(1) ");

        commonQueryHql.append(" FROM MesShippingOrderManagement as ms ");
        commonQueryHql.append("JOIN MesShippingOrderManagementDetail as momd ");
        commonQueryHql.append("ON ms.id = momd.pid ");
        commonQueryHql.append("WHERE 1=1 ");
        commonQueryHql.append("AND momd.organizeCode = :organizeCode ");
        commonQueryHql.append("AND ms.organizeCode = :organizeCode ");
        commonQueryHql.append("AND momd.isDeleted = :isDeleted ");
        commonQueryHql.append("AND momd.isValid = :isValid ");
        commonQueryHql.append("AND ms.isDeleted = :isDeleted ");
        commonQueryHql.append("AND ms.isValid = :isValid ");

        // 动态添加其他查询条件
        if (!StringUtils.isEmpty(model.getShippingCode())) {
            commonQueryHql.append("AND ms.shippingCode like :shippingCode ");
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            commonQueryHql.append("AND momd.partNo like :partNo ");
        }
        if (!StringUtils.isEmpty(model.getCustPartNo())) {
            commonQueryHql.append("AND momd.custPartNo like :custPartNo ");
        }
        if (!StringUtils.isEmpty(model.getShippingType())) {
            commonQueryHql.append("AND ms.shippingType = :shippingType ");
        }
        if (!StringUtils.isEmpty(model.getCustCode())) {
            commonQueryHql.append("AND ms.custCode like :custCode ");
        }
        if (!StringUtils.isEmpty(model.getStartScanTimeStart()) && !StringUtils.isEmpty(model.getStartScanTimeEnd())) {
            commonQueryHql.append("AND ms.startScanTime between :startScanTimeStart and :startScanTimeEnd ");
        }
        if (!StringUtils.isEmpty(model.getEndScanTimeStart()) && !StringUtils.isEmpty(model.getEndScanTimeEnd())) {
            commonQueryHql.append("AND ms.endScanTime between :endScanTimeStart and :endScanTimeEnd ");
        }
        if (!StringUtils.isEmpty(model.getPublishTimeStart()) && !StringUtils.isEmpty(model.getPublishTimeEnd())) {
            commonQueryHql.append("AND ms.publishTime between :publishTimeStart and :publishTimeEnd ");
        }

        Query dataQuery = entityManager.createQuery(dataQueryHql.append(commonQueryHql).toString(), ShippingReportModel.class);
        setQueryParams(dataQuery, model);
        Query countQuery = entityManager.createQuery(countQueryHql.append(commonQueryHql).toString());
        setQueryParams(countQuery, model);

        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<ShippingReportModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }


    private void setQueryParams(Query dataQuery, ShippingReportModel model) {
        dataQuery.setParameter("organizeCode", model.getOrganizeCode());
        dataQuery.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        dataQuery.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());
        if (!StringUtils.isEmpty(model.getShippingCode())) {
            dataQuery.setParameter("shippingCode", model.getShippingCode() + "%");
        }
        if (!StringUtils.isEmpty(model.getPartNo())) {
            dataQuery.setParameter("partNo", model.getPartNo() + "%");
        }
        if (!StringUtils.isEmpty(model.getCustPartNo())) {
            dataQuery.setParameter("custPartNo", model.getCustPartNo() + "%");
        }
        if (!StringUtils.isEmpty(model.getShippingType())) {
            dataQuery.setParameter("shippingType", model.getShippingType());
        }
        if (!StringUtils.isEmpty(model.getCustCode())) {
            dataQuery.setParameter("custCode", model.getCustCode() + "%");
        }
        if (!StringUtils.isEmpty(model.getStartScanTimeStart()) && !StringUtils.isEmpty(model.getStartScanTimeEnd())) {
            dataQuery.setParameter("startScanTimeStart", model.getStartScanTimeStart());
            dataQuery.setParameter("startScanTimeEnd", model.getStartScanTimeEnd());
        }
        if (!StringUtils.isEmpty(model.getEndScanTimeStart()) && !StringUtils.isEmpty(model.getEndScanTimeEnd())) {
            dataQuery.setParameter("endScanTimeStart", model.getEndScanTimeStart());
            dataQuery.setParameter("endScanTimeEnd", model.getEndScanTimeEnd());
        }
        if (!StringUtils.isEmpty(model.getPublishTimeStart()) && !StringUtils.isEmpty(model.getPublishTimeEnd())) {
            dataQuery.setParameter("publishTimeStart",model.getPublishTimeStart());
            dataQuery.setParameter("publishTimeEnd", model.getPublishTimeEnd());
        }
    }
}
