package cn.estsh.i3plus.ext.mes.apiservice.daoimpl.report;

import cn.estsh.i3plus.ext.mes.apiservice.dao.report.ISuspiciousItemBarcodeReportDao;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.model.report.SuspiciousItemBarcodeReportModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import javax.persistence.EntityManager;
import javax.persistence.Query;
import java.util.List;

/**
 * @Description : 可疑品条码信息报表
 * @Reference :
 * @Author : PZJ
 * @CreateDate : 2024/9/28 19:46
 * @Modify:
 **/
@Service
public class SuspiciousItemBarcodeReportDaoImpl implements ISuspiciousItemBarcodeReportDao {

    @Autowired
    private EntityManager entityManager;

    @Override
    public ListPager<SuspiciousItemBarcodeReportModel> queryReport(SuspiciousItemBarcodeReportModel model, Pager pager) {
        StringBuilder dataQueryHql = new StringBuilder();
        StringBuilder countQueryHql= new StringBuilder();
        StringBuilder commonQueryHql = new StringBuilder();
        dataQueryHql.append("select new ")
                .append(SuspiciousItemBarcodeReportModel.class.getName()).append(" ( ");
        dataQueryHql.append("mpd.id as id, ");
        dataQueryHql.append("mpd.organizeCode as organizeCode, ");
        dataQueryHql.append("mpd.defectLocation as defectLocation, ");
        dataQueryHql.append("mpd.frontBack as frontBack, ");
        dataQueryHql.append("mpd.status as status, ");
        dataQueryHql.append("mpd.defectTypeCode as defectTypeCode, ");
        dataQueryHql.append("mpd.defectTypeName as defectTypeName, ");
        dataQueryHql.append("mp.sourceType as sourceType, ");
        dataQueryHql.append("mp.workCenterCode as workCenterCode, ");
        dataQueryHql.append("mp.sn as sn, ");
        dataQueryHql.append("mp.lotNo as lotNo, ");
        dataQueryHql.append("mp.partNo as partNo, ");
        dataQueryHql.append("mp.partName as partName, ");
        dataQueryHql.append("mp.qty as qty, ");
        dataQueryHql.append("mp.inspectionDate as inspectionDate, ");
        dataQueryHql.append("mp.inspectionStatus as inspectionStatus, ");
        dataQueryHql.append("mp.ncStatus as ncStatus, ");
        dataQueryHql.append("mp.shiftCode as shiftCode, ");
        dataQueryHql.append("mpd.systemSyncStatus as systemSyncStatus, ");
        dataQueryHql.append("mpd.systemSyncDatetime as systemSyncDatetime");
        dataQueryHql.append(") ");

        countQueryHql.append("select count(1) ");
        commonQueryHql.append("FROM MesPartInspectionDetail mpd ");
        commonQueryHql.append("LEFT JOIN MesPartInspection mp ON mpd.pid = mp.id ");
        commonQueryHql.append("WHERE mpd.organizeCode = :organizeCode ");
        commonQueryHql.append("AND mp.organizeCode = :organizeCode ");
        commonQueryHql.append("AND mpd.isDeleted = :isDeleted ");
        commonQueryHql.append("AND mp.isDeleted = :isDeleted ");
        commonQueryHql.append("AND mpd.isValid = :isValid ");
        commonQueryHql.append("AND mp.isValid = :isValid ");


        // like 缺陷位置
        if(!StringUtils.isEmpty(model.getDefectLocation())){
            commonQueryHql.append("AND mpd.defectLocation like :defectLocation ");
        }
        // = 方向
        if(!StringUtils.isEmpty(model.getFrontBack())){
            commonQueryHql.append("AND mpd.frontBack = :frontBack ");
        }
        // = nc状态
        if(!StringUtils.isEmpty(model.getNcStatus())){
            commonQueryHql.append("AND mp.ncStatus = :ncStatus ");
        }
        // like 缺陷代码
        if(!StringUtils.isEmpty(model.getDefectTypeCode())){
            commonQueryHql.append("AND mpd.defectTypeCode like :defectTypeCode ");
        }
        //like 缺陷名称
        if(!StringUtils.isEmpty(model.getDefectTypeName())){
            commonQueryHql.append("AND mpd.defectTypeName like :defectTypeName ");
        }
        //like 产线代码
        if(!StringUtils.isEmpty(model.getWorkCenterCode())){
            commonQueryHql.append("AND mp.workCenterCode like :workCenterCode ");
        }
        //like 条码
        if(!StringUtils.isEmpty(model.getSn())){
            commonQueryHql.append("AND mp.sn like :sn ");
        }
        //like 零件号
        if(!StringUtils.isEmpty(model.getPartNo())){
            commonQueryHql.append("AND mp.partNo like :partNo ");
        }
        //like 零件名称
        if(!StringUtils.isEmpty(model.getPartName())){
            commonQueryHql.append("AND mp.partName like :partName ");
        }
        //范围时间 检验日期
        if (!StringUtils.isEmpty(model.getCreateDateTimeStart()) && !StringUtils.isEmpty(model.getCreateDateTimeEnd())){
            commonQueryHql.append("AND mpd.createDateTimeStart between :createDateTimeStart and :createDateTimeEnd ");
        }
        //= 检验状态
        if (!StringUtils.isEmpty(model.getInspectionStatus())){
            commonQueryHql.append("AND mpd.inspectionStatus = :inspectionStatus ");
        }
        //like 班次代码
        if (!StringUtils.isEmpty(model.getShiftCode())){
            commonQueryHql.append("AND mp.shiftCode like :shiftCode ");
        }
        Query dataQuery = entityManager.createQuery(dataQueryHql.append(commonQueryHql).toString(), SuspiciousItemBarcodeReportModel.class);
        setQueryParams(dataQuery, model);
        Query countQuery = entityManager.createQuery(countQueryHql.append(commonQueryHql).toString());
        setQueryParams(countQuery, model);

        //本次查询满足条件的总数
        int count = Integer.parseInt(String.valueOf(countQuery.getSingleResult()));
        pager = PagerHelper.getPager(pager, count);
        //查询本页数据
        dataQuery.setMaxResults(pager.getPageSize());
        dataQuery.setFirstResult(pager.getCurrentPage() * pager.getPageSize() - pager.getPageSize());
        List<SuspiciousItemBarcodeReportModel> resultList = dataQuery.getResultList();
        return new ListPager<>(resultList, pager);
    }

    private void setQueryParams(Query query, SuspiciousItemBarcodeReportModel model) {
        query.setParameter("organizeCode", model.getOrganizeCode());
        query.setParameter("isDeleted", CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue());
        query.setParameter("isValid", CommonEnumUtil.IS_VAILD.VAILD.getValue());

        // like 缺陷位置
        if(!StringUtils.isEmpty(model.getDefectLocation())){
            query.setParameter("defectLocation", model.getDefectLocation() + "%");
        }
        // =方向
        if(!StringUtils.isEmpty(model.getFrontBack())){
            query.setParameter("frontBack", model.getFrontBack());
        }
        // =nc状态
        if(!StringUtils.isEmpty(model.getNcStatus())){
            query.setParameter("ncStatus", model.getNcStatus());
        }
        // like 缺陷代码
        if(!StringUtils.isEmpty(model.getDefectTypeCode())){
            query.setParameter("defectTypeCode", model.getDefectTypeCode() + "%");
        }
        //like 缺陷名称
        if(!StringUtils.isEmpty(model.getDefectTypeName())){
            query.setParameter("defectTypeName", model.getDefectTypeName() + "%");
        }
        if(!StringUtils.isEmpty(model.getWorkCenterCode())){
            query.setParameter("workCenterCode", model.getWorkCenterCode() + "%");
        }
        if(!StringUtils.isEmpty(model.getSn())){
            query.setParameter("sn", model.getSn() + "%");
        }
        if(!StringUtils.isEmpty(model.getPartNo())){
            query.setParameter("partNo",  model.getPartNo() + "%");
        }
        if(!StringUtils.isEmpty(model.getPartName())){
            query.setParameter("partName", model.getPartName() + "%");
        }
        if (!StringUtils.isEmpty(model.getCreateDateTimeStart()) && !StringUtils.isEmpty(model.getCreateDateTimeEnd())){
            query.setParameter("createDateTimeStart", model.getCreateDateTimeStart());
            query.setParameter("createDateTimeEnd", model.getCreateDateTimeEnd());
        }
        if (!StringUtils.isEmpty(model.getInspectionStatus())){
            query.setParameter("inspectionStatus", model.getInspectionStatus());
        }
        if (!StringUtils.isEmpty(model.getShiftCode())){
            query.setParameter("shiftCode", model.getShiftCode()+"%");
        }
    }
}
