package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;


import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.mes.apiservice.util.DateUtil;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.*;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import cn.hutool.json.JSONObject;
import com.google.common.base.Objects;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.beanutils.BeanUtils;
import org.mvel2.util.Make;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import static cn.estsh.i3plus.mes.apiservice.util.DateUtil.FORMAT_HH_MM;
import static cn.estsh.i3plus.mes.apiservice.util.DateUtil.FORMAT_YYYY_MM_DD;

/**
 * @author castle, 暂时没有客供品移库
 * @version 1.0
 * @date 2021/2/2 16:44
 **/
@Slf4j
@DisallowConcurrentExecution
@Component
@ApiOperation("生成对账数据job")
public class MesAccountJob extends BaseMesScheduleJob {

    @Autowired
    private MesProductRecordLogRepository mesProductRecordLogRepository;

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRepository;

    @Autowired
    private MesEquipmentLogDetailRepository mesEquipmentLogDetailRepository;

    @Autowired
    private MesEquipmentRepository mesEquipmentRepository;

    @Autowired
    private MesMouldMultiCavityRepository mesMouldMultiCavityRepository;

    public MesAccountJob() {
        super(MesAccountJob.class, "生成对账数据job");
    }

    @Override
    @Transactional
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        JSONObject jsonObject = new JSONObject();

        String json = this.getJobParam();
        //String json = "{\"startTime\":\"2024-09-13 07:50:00\",\"endTime\":\"2024-09-19 08:00:00\"}";
        com.alibaba.fastjson.JSONObject reqJson = com.alibaba.fastjson.JSONObject.parseObject(json);
        String startDateTime =  reqJson.getString("startTime");
        String endDateTime = reqJson.getString("endTime");
        if (reqJson == null || (StringUtils.isEmpty(startDateTime) && StringUtils.isEmpty(endDateTime))) {
            Date yesterday = TimeTool.getDateBefore(new Date(), 1);
            startDateTime = FORMAT_YYYY_MM_DD.format(yesterday) + " " + " 07:50:00";
            endDateTime = TimeTool.getToday() + " 08:00:00";
        }

        jsonObject.putIfAbsent("organizeCode", "CK01");
        jsonObject.putIfAbsent("userName", "job");

        String organizeCode = jsonObject.getStr("organizeCode");
        String userName = jsonObject.getStr("userName");
        if (null == organizeCode) {
            log.error("请添加需要对账的工厂代码!");
            return;
        }
        if (null == userName) {
            userName = "JOB";
        }

        //String startDateTime = FORMAT_YYYY_MM_DD.format(yesterday) + " " + " 07:50:00";
        //String endDateTime = TimeTool.getToday() + " 08:00:00";
        //String startDateTime =  "2024-09-14 07:50:00";
        //String endDateTime = "2024-09-15 08:00:00";
        //List<String> list = Stream.of("A88_EOL_Electric_inspection").collect(Collectors.toList());
        DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        //DdlPreparedPack.getInPackList(list, "equipmentCode", ddlPackBean);
        DdlPreparedPack.getNumEqualPack(10, "equipmentType", ddlPackBean);

        List<MesEquipment> equipments = mesEquipmentRepository.findByHqlWhere(ddlPackBean);
        if (CollectionUtils.isEmpty(equipments)) {
            return;
        }
        try {
            for (MesEquipment equipment : equipments) {
                LOGGER.info("正常处理设备编号：{}的设备数据开始", equipment.getEquipmentCode());
                DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(equipment.getEquipmentCode(), "equipmentCode", packBean);

                DdlPreparedPack.getStringSmallerNotEqualPack(endDateTime, "createDatetime", packBean);
                DdlPreparedPack.getStringBiggerPack(startDateTime, "createDatetime", packBean);

                DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.ASC.getValue(), packBean);
                List<MesProductRecordLog> recordLogs = mesProductRecordLogRepository.findByHqlWhere(packBean);
                /*recordLogs = recordLogs.stream().filter(productRecordLog ->(!Objects.equal(productRecordLog.getType(), "record") &&  !Objects.equal(productRecordLog.getType(), "logdetail"))
                        || (Objects.equal(productRecordLog.getType(), "logdetail") && StringUtils.isEmpty(productRecordLog.getPartNo()))).collect(Collectors.toList());
*/
                if (CollectionUtils.isEmpty(recordLogs)) {
                    continue;


                }
                if (equipment.getEquipmentCode().contains("HT")) {
                    String partNo = "";
                    List<MesProductRecordLog> productRecordLogList = new ArrayList<>();

                    DdlPackBean detailPackBean = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getStringEqualPack(equipment.getEquipmentCode(), "equipmentCode", detailPackBean);
                    DdlPreparedPack.getStringLikerPack("InjDataSeries", "equipVariableName", detailPackBean);
                    DdlPreparedPack.getStringEqualPack("192", "quality", detailPackBean);

                    DdlPreparedPack.getStringSmallerNotEqualPack(startDateTime, "createDatetime", detailPackBean);

                    DdlPreparedPack.getOrderBy("createDatetime", CommonEnumUtil.ASC_OR_DESC.DESC.getValue(), detailPackBean);
                    List<MesProductRecordLog> detailLogs = mesProductRecordLogRepository.findByHqlTopWhere(detailPackBean, 1);
                    DdlPackBean mouldMultipackBeanTemp = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getStringEqualPack(detailLogs.get(0).getEquipVariableValue(), "mouldNo", mouldMultipackBeanTemp);
                    List<MesMouldMultiCavity> mouldMultiCavities = mesMouldMultiCavityRepository.findByHqlWhere(mouldMultipackBeanTemp);
                    if (CollectionUtils.isEmpty(mouldMultiCavities)) {
                        continue;
                    } else {
                        List<String> partList = mouldMultiCavities.stream().map(MesMouldMultiCavity::getPartNo).distinct().collect(Collectors.toList());
                        partNo = String.join(",", partList);
                    }

                    for (MesProductRecordLog log : recordLogs) {
                        if (!StringUtils.isEmpty(log.getPartNo()) && log.getEquipVariableName() != null
                                && !log.getEquipVariableName().contains("InjDataSeries")) {
                            continue;
                        }
                        if (log.getType().equals("logdetail") && log.getEquipVariableName().contains("InjDataSeries")) {
                            if (StringUtils.isEmpty(log.getEquipVariableValue())) {
                                continue;
                            }
                            DdlPackBean mouldMultipackBean = DdlPackBean.getDdlPackBean(organizeCode);
                            DdlPreparedPack.getStringEqualPack(log.getEquipVariableValue(), "mouldNo", mouldMultipackBean);
                            List<MesMouldMultiCavity> mesMouldMultiCavities = mesMouldMultiCavityRepository.findByHqlWhere(mouldMultipackBean);
                            if (CollectionUtils.isEmpty(mesMouldMultiCavities)) {
                                continue;
                            } else {
                                List<String> partList = mesMouldMultiCavities.stream().map(MesMouldMultiCavity::getPartNo).distinct().collect(Collectors.toList());
                                partNo = String.join(",", partList);
                                log.setPartNo(partNo);
                            }
                        } else {
                            log.setPartNo(partNo);
                            if (!StringUtils.isEmpty(partNo)) {
                                productRecordLogList.add(log);
                            }
                        }
                    }
                    mesProductRecordLogRepository.saveAll(productRecordLogList);
                } else {
                    String partNo = "";
                    List<MesProductRecordLog> productRecordLogs = new ArrayList<>();
                    for (MesProductRecordLog log : recordLogs) {

                        if (log.getType().equals("monitor") && !StringUtils.isEmpty(log.getPartNo())) {
                            partNo = log.getPartNo();
                        } else {

                            log.setPartNo(partNo);
                            if (!StringUtils.isEmpty(partNo)) {
                                productRecordLogs.add(log);
                            }

                        }
                    }
                    mesProductRecordLogRepository.saveAll(productRecordLogs);
                }
                LOGGER.info("正常处理设备编号：{}的设备数据结束");
            }
        }catch (Exception e) {
            e.printStackTrace();
        }
        //
    }
}

