package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesPullingOrderInfoService;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;

import java.util.List;

/**
 * @Description : 根据拉动组和生产工单产生拉动单JOB
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/9/4 10:01
 * @Modify:
 **/
// 禁止 JOB 并发执行
@DisallowConcurrentExecution
@ApiOperation("根据拉动组和生产工单产生拉动单JOB")
public class MesCreatePullOrderByPullGroupAndWorkOrderJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesCreatePullOrderByPullGroupAndWorkOrderJob.class);

    private static final long serialVersionUID = -321526601595924818L;

    @Autowired
    private IMesPullingOrderInfoService pullingOrderInfoService;

    @Autowired
    private IMesConfigService mesConfigService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesCreatePullOrderByPullGroupAndWorkOrderJob() {
        super(MesCreatePullOrderByPullGroupAndWorkOrderJob.class, "根据拉动组和生产工单产生拉动单JOB");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("job参数为空，请检查参数");
            }
            String[] organizeCodeList = jobParam.split(",");
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB开始 -----start");
            String userName = "CreatePullOrderJob";
            for (String organizeCode : organizeCodeList) {
                pullingOrderInfoService.doCreatePullOrderByPullGroupAndWorkOrder(organizeCode,userName);
            }
            LOGGER.info("根据拉动组和生产工单产生拉动单JOB开始结束 ----- end");

        } catch (Exception e) {
            LOGGER.error("根据拉动组和生产工单产生拉动单JOB任务结束e:{}", e.toString());
        }
    }

}
