package cn.estsh.i3plus.ext.mes.apiservice.schedulejob;

import cn.estsh.i3plus.ext.mes.api.base.IMesCcscTaskService;
import cn.estsh.i3plus.ext.mes.api.base.IMesDowntimeRecordService;
import cn.estsh.i3plus.ext.mes.api.base.IMesWorkOrderService;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesBasDowntimeReasonTypeServiceImpl;
import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base.MesDowntimeRecordServiceImpl;
import cn.estsh.i3plus.mes.apiservice.schedulejob.BaseMesScheduleJob;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrder;
import cn.estsh.i3plus.pojo.mes.repository.IMesBasDowntimeReasonRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.impp.framework.boot.init.ApplicationProperties;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.quartz.DisallowConcurrentExecution;
import org.quartz.JobExecutionContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description : MES设备生产停机记录
 * @Reference :
 * @Author : gsz
 * @CreateDate 2024/5/28 10:58
 * @Modify:
 **/
@DisallowConcurrentExecution
@ApiOperation("MES设备生产停机记录")
public class MesDowntimeRecordJob extends BaseMesScheduleJob {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesDowntimeRecordJob.class);

    private static final long serialVersionUID = 964832621370364379L;

    @Autowired
    private MesEquipmentRepository mesEquipmentRDao;

    @Autowired
    private IMesDowntimeRecordService downtimeRecordService;

    @Value("${server.port}")
    private String serverPort;
    @Value("${impp.server.ip}")
    private String ip;

    public MesDowntimeRecordJob() {
        super(MesDowntimeRecordJob.class, "MES设备生产停机记录");
        this.setMultiInstance(true);
    }

    @Override
    public void executeMesJob(JobExecutionContext context, ApplicationProperties applicationProperties) {
        try {
            String jobParam = this.getJobParam();

            if (StringUtils.isBlank(jobParam)) {
                throw new IllegalArgumentException("job参数为空，请检查参数");
            }
            String[] organizeCodeList = jobParam.split(",");
            LOGGER.info("MES设备生产停机记录 -----start");

            for (String organizeCode : organizeCodeList) {

                List<MesEquipment> mesEquipmentList = mesEquipmentRDao.findByProperty(
                        new String[]{MesConstWords.ORGANIZE_CODE,"monitorDown", MesConstWords.IS_DELETED, MesConstWords.IS_VALID},
                        new Object[]{organizeCode,CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                                CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                                CommonEnumUtil.IS_VAILD.VAILD.getValue()});
                if (!CollectionUtils.isEmpty(mesEquipmentList)){
                    downtimeRecordService.doMesDowntimeRecordJob(organizeCode,mesEquipmentList);
                }
            }
            LOGGER.info("MES设备生产停机记录 ----- end");

        } catch (Exception e) {
            LOGGER.error("MES设备生产停机记录 e:{}",e.toString());
        }
    }

}
