package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesConfigService;
import cn.estsh.i3plus.ext.mes.api.base.IMesDowntimeRecordService;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.platform.common.exception.ImppExceptionEnum;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.repository.MesDowntimeRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesProductionRecordRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWcEquipmentRepository;
import cn.estsh.impp.framework.boot.exception.ImppExceptionBuilder;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Objects;


/**
 * @Description: 设备生产停机记录
 * @Author: gsz
 * @Date: 2024/5/27 19:10
 * @Modify:
 */
@Service
@Slf4j
public class MesDowntimeRecordServiceImpl extends BaseMesService<MesDowntimeRecord> implements IMesDowntimeRecordService {

    @Autowired
    private MesProductionRecordRepository mesProductionRecordRDao;

    @Autowired
    private MesDowntimeRecordRepository mesDowntimeRecordRDao;

    @Autowired
    private IMesConfigService mesConfigService;

    @Autowired
    private MesWcEquipmentRepository wcEquipmentRepository;
    @Override
    protected void setPackQueryBean(MesDowntimeRecord bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getReasonCode(), "reasonCode", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getReasonTypeCode(), "reasonTypeCode", packBean);
    }

    @Override
    protected void onInsertBean(MesDowntimeRecord item) {
        // 数据校验
        ValidatorBean.checkNotNull(item.getOrganizeCode(), "工厂号不能为空");
        ValidatorBean.checkNotNull(item.getEquipmentCode(), "equipmentCode不能为空");
        ValidatorBean.checkNotNull(item.getReasonCode(), "reasonCode不能为空");
        ValidatorBean.checkNotNull(item.getReasonTypeCode(), "reasonTypeCode不能为空");

        //唯一性校验
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(item.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getReasonCode(), "reasonCode", packBean);
        DdlPreparedPack.getStringEqualPack(item.getReasonTypeCode(), "reasonTypeCode", packBean);
        boolean flg = baseRDao.isExitByHql(packBean);
        if (flg) {
            throw ImppExceptionBuilder.newInstance()
                    .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
                    .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
                    .setErrorDetail("设备生产停机记录设备【%s】停机原因类型【%s】停机原因代码【%s】已经存在，请检查数据",
                            item.getEquipmentCode(), item.getReasonTypeCode(), item.getReasonCode())
                    .build();
        }
    }

    @Override
    public void doMesDowntimeRecordJob(String organizeCode, List<MesEquipment> mesEquipmentList) {
        MesConfig reasonCfg = mesConfigService.getCfgValueByCode(organizeCode, MesExtConstWords.MES_DOWNTIME_RECORD_JOB_REASON);
        String reason = "";
        if (!Objects.isNull(reasonCfg) && !StringUtils.isEmpty(reasonCfg.getCfgValue())) {
            reason = reasonCfg.getCfgValue();
        }
        MesConfig typeCfg = mesConfigService.getCfgValueByCode(organizeCode, MesExtConstWords.MES_DOWNTIME_RECORD_JOB_TYPE);
        String type = "";
        if (!Objects.isNull(typeCfg) && !StringUtils.isEmpty(typeCfg.getCfgValue())) {
            type = typeCfg.getCfgValue();
        }

        for (MesEquipment mesEquipment : mesEquipmentList) {
            //字段是否为“是” 且监控停机时间不等于空和0 才继续
            if (mesEquipment.getMonitorDown() != 1 || StringUtil.isEmpty(mesEquipment.getMonitorDownTime()) || Integer.parseInt(mesEquipment.getMonitorDownTime()) == 0) {
                continue;
            }
            Integer monitorDownTime = Integer.parseInt(mesEquipment.getMonitorDownTime());
            String previousDay = getPreviousDay(new Date(), monitorDownTime);
            DdlPackBean ddlPackBean = DdlPackBean.getDdlPackBean(organizeCode);
            DdlPreparedPack.getStringEqualPack(mesEquipment.getEquipmentCode(), "equipmentCode", ddlPackBean);
            DdlPreparedPack.getStringBiggerPack(previousDay, "completeDateTime", ddlPackBean);//todo
//            DdlPreparedPack.getNumberSmallerEqualPack(mesEquipment.getCreateDateTimeEnd(), "createDatetime", ddlPackBean);
            MesProductionRecord mesProductionRecord = mesProductionRecordRDao.getByProperty(ddlPackBean);
            if (StringUtil.isEmpty(mesProductionRecord)) {
                DdlPackBean ddlPackBeanDowntime = DdlPackBean.getDdlPackBean(organizeCode);
                DdlPreparedPack.getStringEqualPack(mesEquipment.getEquipmentCode(), "equipmentCode", ddlPackBeanDowntime);
                if(!StringUtil.isEmpty(reason)){
                    DdlPreparedPack.getStringEqualPack(reason, "reasonCode", ddlPackBeanDowntime);
                }
                if(!StringUtil.isEmpty(type)){
                    DdlPreparedPack.getStringEqualPack(type, "reasonTypeCode", ddlPackBeanDowntime);
                }
                MesDowntimeRecord mesDowntimeRecordDao = mesDowntimeRecordRDao.getByProperty(ddlPackBeanDowntime);
                if (StringUtil.isEmpty(mesDowntimeRecordDao)) {
                    MesWcEquipment wcEquipmentDbList = wcEquipmentRepository.getByProperty(
                            new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, MesConstWords.IS_VALID,""},
                            new Object[]{organizeCode, CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(), CommonEnumUtil.IS_VAILD.VAILD.getValue(),
                                    mesEquipment.getEquipmentCode()});
                    //新增设备停机记录
                    MesDowntimeRecord mesDowntimeRecord = new MesDowntimeRecord();
                    mesDowntimeRecord.setOrganizeCode(organizeCode);
                    mesDowntimeRecord.setEquipId(mesEquipment.getEquipId());
                    mesDowntimeRecord.setEquipmentCode(mesEquipment.getEquipmentCode());
                    mesDowntimeRecord.setAreaCode(StringUtil.isEmpty(wcEquipmentDbList)?"":wcEquipmentDbList.getAreaCode());
                    mesDowntimeRecord.setWorkCenterCode(StringUtil.isEmpty(wcEquipmentDbList)?"":wcEquipmentDbList.getWorkCenterCode());
                    mesDowntimeRecord.setWorkCellCode(StringUtil.isEmpty(wcEquipmentDbList)?"":wcEquipmentDbList.getWorkCellCode());
                    mesDowntimeRecord.setReasonCode(StringUtil.isEmpty(reason)?"21":reason);
                    mesDowntimeRecord.setReasonTypeCode(StringUtil.isEmpty(type)?"6":type);
                    ConvertBean.serviceModelInitialize(mesDowntimeRecord, "MesDowntimeRecordJob");
                    //将当前新增数据的create_date_time时间些到上一条记录的modify_date_time字段
                    DdlPackBean ddlPackBeanOt = DdlPackBean.getDdlPackBean(organizeCode);
                    DdlPreparedPack.getStringEqualPack(mesEquipment.getEquipmentCode(), "equipmentCode", ddlPackBeanOt);
                    ddlPackBeanOt.setWhereAppend(ddlPackBeanOt.getWhereAppend() + " order by createDatetime desc ");
                    MesDowntimeRecord mesDowntimeRecordOt = mesDowntimeRecordRDao.getByProperty(ddlPackBeanOt);
                    if (!StringUtil.isEmpty(mesDowntimeRecordOt)){
                        mesDowntimeRecordOt.setModifyDatetime(mesDowntimeRecord.getCreateDatetime());
                        mesDowntimeRecordOt.setModifyUser("MesDowntimeRecordJob");
                        mesDowntimeRecordRDao.update(mesDowntimeRecordOt);
                    }
                    mesDowntimeRecordRDao.insert(mesDowntimeRecord);

                }else {
                    ConvertBean.serviceModelUpdate(mesDowntimeRecordDao, "MesDowntimeRecordJob");
                    mesDowntimeRecordRDao.update(mesDowntimeRecordDao);
                }
            }

        }
    }

    /**
     * 获取某个时间的前几分钟
     *
     * @return yyyy-MM-dd HH:mm:ss
     */
    public static String getPreviousDay(Date dateTime, int amount) {
        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        Calendar c = Calendar.getInstance();
        if (dateTime != null) {
            c.setTime(dateTime);
        }
        c.add(Calendar.MINUTE, -amount);
        Date start = c.getTime();
        String date = format.format(start);//前几分钟
        return date;
    }
}
