package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesEquipmentVariableService;
import cn.estsh.i3plus.ext.mes.apiservice.config.AuthUtilExt;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.constant.MesCommonConstant;
import cn.estsh.i3plus.mes.api.iservice.busi.ISyncFuncService;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipment;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentChannel;
import cn.estsh.i3plus.pojo.mes.bean.MesEquipmentVariable;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentChannelRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesEquipmentRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import cn.estsh.impp.framework.boot.util.ValidatorBean;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.*;

/**
 * @Description :SAP生产计划
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/5/6 15:52
 * @Modify:
 **/
@Service
@Slf4j
public class MesEquipmentVariableService extends BaseMesService<MesEquipmentVariable> implements IMesEquipmentVariableService {

    public static final Logger LOGGER = LoggerFactory.getLogger(MesEquipmentVariableService.class);


    @Autowired
    private MesEquipmentRepository equipmentRepository;

    @Autowired
    private MesEquipmentChannelRepository equipmentChannelRepository;

    @Autowired
    private ISyncFuncService syncFuncService;

    protected void setPackQueryBean(MesEquipmentVariable bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getEquipmentCode(), "equipmentCode", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getCode(), "code", packBean);
    }

    @Override
    protected void onInsertBean(MesEquipmentVariable item) {
        item.setClientHandle(Integer.parseInt(getClientHandleId(1, item.getOrganizeCode(), item.getCreateUser()).get(0).replace(MesCommonConstant.CLIENT_HANDLE_ID,"")));
        MesEquipment mesEquipment = getMesEquipment(item.getOrganizeCode(),item.getEquipmentCode());
        item.setEquipId(mesEquipment.getEquipId());
    }

    @Override
    public void validateImport(List<MesEquipmentVariable> beanList) {

        for (MesEquipmentVariable item : beanList) {
            ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");

//            DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(item.getOrganizeCode());
//            DdlPreparedPack.getStringEqualPack(item.getEquipmentCode(), "equipmentCode", seriesPackBean);
//            boolean flg = baseRDao.isExitByHql(seriesPackBean);
//            if (flg) {
//                throw ImppExceptionBuilder.newInstance()
//                        .setSystemID(CommonEnumUtil.SOFT_TYPE.MES.getCode())
//                        .setErrorCode(ImppExceptionEnum.VARIFY_EXCEPTION.getCode())
//                        .setErrorDetail("【%s】设备代码已经存在，请检查数据", item.getEquipmentCode())
//                        .build();
//            }
        }
    }

    @Override
    public void addMesEquipmentVariable(List<Map<String, Object>> mapList,String equipmentCode, String userName, String organizeCode, String device) {
        MesEquipment mesEquipment = getMesEquipment(organizeCode,equipmentCode);
        MesEquipmentChannel mesEquipmentChannel = getMesEquipmentChannel(organizeCode,equipmentCode);

        List<MesEquipmentVariable> mesEquipmentVariables = new ArrayList<>();
        for (Map<String, Object> objectMap : mapList) {
            List<Object> objects = Arrays.asList(objectMap.values().toArray());
            try {
                MesEquipmentVariable mesEquipmentVariable = new MesEquipmentVariable();
                mesEquipmentVariable.setEquipId(mesEquipment.getEquipId());
                mesEquipmentVariable.setChannel(mesEquipmentChannel.getChannel());
                mesEquipmentVariable.setEquipmentCode(mesEquipment.getEquipmentCode());
                mesEquipmentVariable.setTagName(String.valueOf(objectMap.get("Tag Name")));
                mesEquipmentVariable.setTagAddress(String.valueOf(objectMap.get("Address")));
                mesEquipmentVariable.setCode(String.valueOf(objectMap.get("Address")));
                if ("R/W".equals(objectMap.get("Client Access").toString())) {
                    mesEquipmentVariable.setReadWriteFlag(MesExtEnumUtil.READ_WRITE_FLAG.READ_WRITE.getValue());
                } else if ("RO".equals(objectMap.get("Client Access").toString())) {
                    mesEquipmentVariable.setReadWriteFlag(MesExtEnumUtil.READ_WRITE_FLAG.READ.getValue());
                }
                String dataType = String.valueOf(objectMap.get("Data Type"));
                if ("Boolean".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_40.getValue());
                } else if ("Byte".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_30.getValue());
                } else if ("Byte".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_30.getValue());
                } else if ("Byte Array".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_30.getValue());
                } else if ("Double".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_70.getValue());
                } else if ("Double Array".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_70.getValue());
                } else if ("DWord".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_50.getValue());
                } else if ("Float".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_60.getValue());
                } else if ("Float Array".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_60.getValue());
                } else if ("Long".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_15.getValue());
                } else if ("Long Array".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.ARIABLE_DATA_TYPE_15.getValue());
                } else if ("Short".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_10.getValue());
                } else if ("Short Array".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_10.getValue());
                } else if ("String".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_50.getValue());
                } else if ("Word".equals(dataType)) {
                    mesEquipmentVariable.setDataType(MesExtEnumUtil.VARIABLE_DATA_TYPE.VARIABLE_DATA_TYPE_50.getValue());
                }

                mesEquipmentVariable.setVariableType(Integer.parseInt(String.valueOf(objectMap.get("Respect Data Type"))));
                mesEquipmentVariable.setScanInterval(Integer.valueOf(String.valueOf(objectMap.get("Scan Rate"))));
                mesEquipmentVariable.setDescription(String.valueOf(objectMap.get("Description")));
                mesEquipmentVariable.setOrganizeCode(organizeCode);
                mesEquipmentVariable.setDevice(device);
                ConvertBean.serviceModelInitialize(mesEquipmentVariable, userName);
                mesEquipmentVariables.add(mesEquipmentVariable);
            } catch (NumberFormatException e) {
                log.info("设备变量导入异常：" + objects);
            }
        }
        insertBatch(mesEquipmentVariables);
    }

    @Override
    public List<MesEquipmentVariable> validateReturnImport(List<MesEquipmentVariable> beanList) {
        String organizeCode = AuthUtilExt.getOrganizeCode();
        String userName = AuthUtilExt.getUserName();
        if(CollectionUtils.isEmpty(beanList)) return beanList;
        List<String> resultList = getClientHandleId(beanList.size(), organizeCode, userName);
        int i = 1;
        for (MesEquipmentVariable item : beanList) {
            // 数据校验
            if(StringUtil.isEmpty(item.getEquipmentCode())){
                LOGGER.info("第{}行,设备代码不能为空", i);
                i++;
                continue;
            }
            ValidatorBean.checkNotNull(item.getEquipmentCode(), "设备代码不能为空");

            MesEquipment itemFlg = getMesEquipment(item.getOrganizeCode(),item.getEquipmentCode());
            item.setEquipId(itemFlg.getEquipId());
            item.setClientHandle(Integer.parseInt(resultList.get(i-1).replace(MesCommonConstant.CLIENT_HANDLE_ID,"")));
            i++;
        }
        return beanList;
    }

    private List<String> getClientHandleId(int size, String organizeCode, String userName) {
        GenSerialNoModel genSerialNoModel = new GenSerialNoModel(MesCommonConstant.CLIENT_HANDLE_ID);
        List<String> resultList = syncFuncService.syncSerialNo(genSerialNoModel.organizeCode(organizeCode), userName, organizeCode, size).getResultList();
        return resultList;
    }

    private MesEquipment getMesEquipment(String organizeCode,String equipmentCode) {
        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", seriesPackBean);
        MesEquipment mesEquipment = equipmentRepository.getByProperty(seriesPackBean);
        if(Objects.isNull(mesEquipment)){
            MesException.throwMesBusiException("设备代码【%s】信息不存在",equipmentCode);
        }
        return mesEquipment;
    }
    private MesEquipmentChannel getMesEquipmentChannel(String organizeCode,String equipmentCode) {
        DdlPackBean seriesPackBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(equipmentCode, "equipmentCode", seriesPackBean);
        MesEquipmentChannel mesEquipmentChannel = equipmentChannelRepository.getByProperty(seriesPackBean);
        if(Objects.isNull(mesEquipmentChannel)){
            MesException.throwMesBusiException("设备通道信息【%s】信息不存在",equipmentCode);
        }
        return mesEquipmentChannel;
    }
}
