package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesPartInspectionService;
import cn.estsh.i3plus.platform.common.util.MesConstWords;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.base.util.StringUtil;
import cn.estsh.i3plus.pojo.mes.bean.MesOrg;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkCenter;
import cn.estsh.i3plus.pojo.mes.bean.nc.MesPartInspection;
import cn.estsh.i3plus.pojo.mes.repository.MesOrgRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesWorkCenterRepository;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.shiro.util.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Arrays;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesPartInspectionService extends BaseMesService<MesPartInspection> implements IMesPartInspectionService {
    @Autowired
    private MesWorkCenterRepository mesWorkCenterRDao;

    @Autowired
    private MesOrgRepository mesOrgRDao;
    @Override
    protected void setPackQueryBean(MesPartInspection bean, DdlPackBean packBean) {
        DdlPreparedPack.getStringEqualPack(bean.getSn(), "sn", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getNcStatus(), "ncStatus", packBean);
        if (StringUtils.isNotBlank(bean.getCreateDateTimeStart()) && StringUtils.isNotBlank(bean.getCreateDateTimeEnd())) {
            DdlPreparedPack.timeBuilder(bean.getCreateDateTimeStart(), bean.getCreateDateTimeEnd(), "createDatetime", packBean, true);
        }
        if (StringUtils.isNotBlank(bean.getModifyDateTimeStart()) && StringUtils.isNotBlank(bean.getModifyDateTimeEnd())) {
            DdlPreparedPack.timeBuilder(bean.getModifyDateTimeStart(), bean.getModifyDateTimeEnd(), "modifyDatetime", packBean, true);
        }
//        DdlPreparedPack.getStringLikerPack(bean.getInspectionDate(), "inspectionDate", packBean);
    }

    @Override
    public ListPager queryMesPartInspectionParamByPager(MesPartInspection bean, Pager pager) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
        DdlPreparedPack.getStringEqualPack(bean.getSn(), "sn", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartName(), "partName", packBean);
        //3.增加产线查询条件，选用下拉框，支持多选
        if(!StringUtil.isEmpty(bean.getWorkCenterCode())){
            DdlPreparedPack.getInPackList(Arrays.asList(bean.getWorkCenterCode().split(",")), "workCenterCode", packBean);
        }
//        DdlPreparedPack.getStringLikerPack(bean.getWorkCenterCode(), "workCenterCode", packBean);
        DdlPreparedPack.getNumEqualPack(bean.getNcStatus(), "ncStatus", packBean);
        if (StringUtils.isNotBlank(bean.getCreateDateTimeStart()) && StringUtils.isNotBlank(bean.getCreateDateTimeEnd())) {
            DdlPreparedPack.timeBuilder(bean.getCreateDateTimeStart(), bean.getCreateDateTimeEnd(), "createDatetime", packBean, true);
        }
        if (StringUtils.isNotBlank(bean.getModifyDateTimeStart()) && StringUtils.isNotBlank(bean.getModifyDateTimeEnd())) {
            DdlPreparedPack.timeBuilder(bean.getModifyDateTimeStart(), bean.getModifyDateTimeEnd(), "modifyDatetime", packBean, true);
        }
        DdlPackBean packBeanLo = DdlPackBean.getDdlPackBean(bean.getOrganizeCode());
//        DdlPreparedPack.getStringLikerPack(bean.getInventoryLocationCode(), "responsibleParty", packBeanLo);
        List<MesOrg> orgList = mesOrgRDao.findByHqlWhere(packBeanLo);
        //2.增加库区查询条件，文本框，支持模糊输入
//        LOGGER.info("queryMesPartInspectionParamByPager-orgList:{}", orgList);
        if(!StringUtil.isEmpty(bean.getInventoryLocationCode())){
            if (!CollectionUtils.isEmpty(orgList)){
                String inventoryLocationCode = bean.getInventoryLocationCode();
                List<String> collect = orgList.stream().filter(o->o.getInventoryLocationCode().equals(inventoryLocationCode))
                        .map(MesOrg::getResponsibleParty).collect(Collectors.toList());
                DdlPreparedPack.getInPackList(collect,"responsibleParty", packBean);
            }
        }
        pager = PagerHelper.getPager(pager, baseRDao.findByHqlWhereCount(packBean));

        List<MesPartInspection> resultList = baseRDao.findByHqlWherePage(packBean, pager);
        if (!CollectionUtils.isEmpty(resultList)){
            for (MesPartInspection mesPartInspection : resultList) {
                if (!StringUtils.isEmpty(mesPartInspection.getWorkCenterCode())){
                    MesWorkCenter mesWorkCenter = mesWorkCenterRDao.getByProperty(
                            new String[]{MesConstWords.ORGANIZE_CODE, MesConstWords.IS_DELETED, MesConstWords.IS_VALID, "workCenterCode"},
                            new Object[]{bean.getOrganizeCode(), CommonEnumUtil.TRUE_OR_FALSE.FALSE.getValue(),
                                    CommonEnumUtil.IS_VAILD.VAILD.getValue(), mesPartInspection.getWorkCenterCode()});
                    if (null != mesWorkCenter) {
                        mesPartInspection.setWorkCenterName(mesWorkCenter.getWorkCenterName());
                    }
                }
                //1.页面增加库区字段（库区根据报废选择的责任人关联查询）
                if (!StringUtils.isEmpty(mesPartInspection.getResponsibleParty())){
                    if (!CollectionUtils.isEmpty(orgList)){
                        List<MesOrg> collect = orgList.stream().filter(a -> mesPartInspection.getResponsibleParty().equals(a.getResponsibleParty())).collect(Collectors.toList());
                        if (!CollectionUtils.isEmpty(collect)){
                            mesPartInspection.setInventoryLocationCode(collect.get(0).getInventoryLocationCode());
                        }
                    }
                }
            }
        }
        return new ListPager<>(resultList, pager);
    }
}
