package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProduceSnService;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.ext.mes.pojo.util.OverwriteStringJoin;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Comparator;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @Description : 产品条码表
 * @Reference :
 * @Author : junsheng.li
 * @CreateDate 2024/6/28 9:27
 * @Modify:
 **/
@Service
@Slf4j
public class MesProduceSnServiceImpl extends BaseMesService<MesProduceSn> implements IMesProduceSnService {

    @Override
    public List<MesProduceSn> findMesProduceSn(String organizeCode, String sn) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getStringEqualPack(sn, "productSn", packBean);
        List<MesProduceSn> produceSnList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(produceSnList)) {
            MesException.throwMesBusiException("条码【%s】信息不存在", sn);
        }
        return produceSnList.stream().filter(o -> null != o).sorted(Comparator.comparing(MesProduceSn::getCreateDatetime).reversed()).collect(Collectors.toList());
    }

    @Override
    public List<MesProduceSn> findMesProduceSnList(String organizeCode, List<String> snList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackList(snList, "productSn", packBean);
        List<MesProduceSn> produceSnList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(produceSnList)) {
            MesException.throwMesBusiException("条码【%s】信息不存在", OverwriteStringJoin.join(MesExtConstWords.COMMA, snList));
        }
        return produceSnList.stream().filter(Objects::nonNull).sorted(Comparator.comparing(MesProduceSn::getCreateDatetime).reversed()).collect(Collectors.toList());
    }
}
