package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.base;

import cn.estsh.i3plus.ext.mes.api.base.IMesProduceSnService;
import cn.estsh.i3plus.ext.mes.api.base.IMesShippingOrderManagementDetailService;
import cn.estsh.i3plus.ext.mes.apiservice.dao.IMesShippingOrderManagementDetailDao;
import cn.estsh.i3plus.ext.mes.apiservice.utils.MesException;
import cn.estsh.i3plus.ext.mes.pojo.model.MesShippingOrderManagementDetailModel;
import cn.estsh.i3plus.platform.common.convert.ConvertBean;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.bean.ListPager;
import cn.estsh.i3plus.pojo.base.common.Pager;
import cn.estsh.i3plus.pojo.base.common.PagerHelper;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.MesPartShippingGroupDetail;
import cn.estsh.i3plus.pojo.mes.bean.MesProduceSn;
import cn.estsh.i3plus.pojo.mes.bean.MesWorkOrderPart;
import cn.estsh.i3plus.pojo.mes.bean.shipping.MesShippingOrderManagementDetail;
import cn.estsh.i3plus.pojo.mes.repository.MesProduceSnRepository;
import cn.estsh.i3plus.pojo.mes.repository.MesShippingOrderManagementDetailRepository;
import cn.estsh.i3plus.pojo.mes.util.MesExtEnumUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

@Service
@Slf4j
public class MesShippingOrderManagementDetailService extends BaseMesService<MesShippingOrderManagementDetail> implements IMesShippingOrderManagementDetailService {

    @Autowired
    private IMesShippingOrderManagementDetailDao shippingOrderManagementDetailDao;

    @Autowired
    private MesShippingOrderManagementDetailRepository shippingOrderDetailRepository;

    @Autowired
    private IMesProduceSnService mesProduceSnService;

    @Autowired
    private MesProduceSnRepository mesProduceSnRepository;

    protected void setPackQueryBean(MesShippingOrderManagementDetail bean, DdlPackBean packBean) {
        DdlPreparedPack.getNumEqualPack(bean.getPid(), "pid", packBean);
        DdlPreparedPack.getStringLikerPack(bean.getPartNo(), "partNo", packBean);
        DdlPreparedPack.getStringEqualPack(bean.getCustOrderNo(), "custOrderNo", packBean);
        DdlPreparedPack.getOrderByPack(new Object[]{CommonEnumUtil.ASC_OR_DESC.DESC.getValue()}, new String[]{"custInfoSeq"}, packBean);
    }

    public ListPager<MesShippingOrderManagementDetailModel> queryDetailGroupByPartNo(MesShippingOrderManagementDetail bean, Pager pager) {

        int count = shippingOrderManagementDetailDao.queryOrderDetailGroupByPartCount(bean);

        pager = PagerHelper.getPager(pager, count);

        List<MesShippingOrderManagementDetailModel> modelList = shippingOrderManagementDetailDao.queryOrderDetailGroupByPartInfo(bean, pager);

        return new ListPager<>(modelList, pager);
    }

    @Override
    public ListPager<MesShippingOrderManagementDetailModel> queryMesShippingDetailsByPager(MesShippingOrderManagementDetailModel mesShippingOrderManagementDetailModel, Pager pager) {
        return shippingOrderManagementDetailDao.queryMesShippingDetailsByPager(mesShippingOrderManagementDetailModel, pager);
    }

    @Override
    public void doShippingBack(Long[] idList, String organizeCode, String userName) {
        List<MesShippingOrderManagementDetail> detailList = getManagementDetails(idList, organizeCode, userName);
        if (CollectionUtils.isEmpty(detailList)) MesException.throwMesBusiException("选择数据无效,请检查");
        Map<String, List<MesProduceSn>> mesProduceSnMap = getMesProduceSnMap(organizeCode, detailList);
        List<MesProduceSn> updateMesProduceSn = new ArrayList<>();
        for (MesShippingOrderManagementDetail detail : detailList) {
            if (!mesProduceSnMap.containsKey(detail.getBarcode()))
                MesException.throwMesBusiException("发运单条码[%s]信息不存在", detail.getBarcode());
            //更新条码状态为完成
            MesProduceSn mesProduceSn = mesProduceSnMap.get(detail.getBarcode()).iterator().next();
            mesProduceSn.setSnStatus(MesExtEnumUtil.PRODUCE_SN_STATUS.OFFLINE.getValue());
            mesProduceSn.setSystemSyncStatus(CommonEnumUtil.FALSE);
            ConvertBean.serviceModelUpdate(mesProduceSn, userName);
            updateMesProduceSn.add(mesProduceSn);
            //软删明细
            detail.setIsDeleted(CommonEnumUtil.VALID);
            detail.setSystemSyncStatus(CommonEnumUtil.FALSE);
            ConvertBean.serviceModelUpdate(detail, userName);
        }
        baseRDao.saveAll(detailList);
        if (!updateMesProduceSn.isEmpty()) {
            mesProduceSnRepository.saveAll(updateMesProduceSn);
        }
    }

    private Map<String, List<MesProduceSn>> getMesProduceSnMap(String organizeCode, List<MesShippingOrderManagementDetail> detailList) {
        List<String> snList = detailList.stream().map(MesShippingOrderManagementDetail::getBarcode).collect(Collectors.toList());
        if (CollectionUtils.isEmpty(snList)) MesException.throwMesBusiException("发运单条码信息不存在");
        List<MesProduceSn> mesProduceSnList = mesProduceSnService.findMesProduceSnList(organizeCode, snList);
        return mesProduceSnList.stream().collect(Collectors.groupingBy(MesProduceSn::getProductSn));
    }

    private List<MesShippingOrderManagementDetail> getManagementDetails(Long[] idList, String organizeCode, String userName) {
        if (Objects.isNull(idList) || StringUtils.isEmpty(organizeCode) || StringUtils.isEmpty(userName)) return null;
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(organizeCode);
        DdlPreparedPack.getInPackArray(idList, "id", packBean);
        List<MesShippingOrderManagementDetail> detailList = baseRDao.findByHqlWhere(packBean);
        if (CollectionUtils.isEmpty(detailList) || detailList.size() <= 0)
            MesException.throwMesBusiException("选择数据不存在,请检查数据");
        return detailList;
    }

    public ListPager<MesWorkOrderPart> queryDetailMesWorkOrderPart(MesWorkOrderPart bean, Pager pager) {
        List<MesWorkOrderPart> modelList=new ArrayList<>();
        List<MesShippingOrderManagementDetail> shippingDetails = shippingOrderManagementDetailDao.queryMesShippingOrderManagementDetail(bean);
        if (CollectionUtils.isEmpty(shippingDetails)){
            return new ListPager<>(modelList, pager);
        }
        bean.setShippingDetails(shippingDetails);
        int count = shippingOrderManagementDetailDao.queryDetailGroupByPartNo(bean);

        pager = PagerHelper.getPager(pager, count);

        modelList = shippingOrderManagementDetailDao.queryOrderDetailGroupByPartInfo(bean, pager);
        //查询发运明细
        if (!CollectionUtils.isEmpty(modelList)){
            Map<String, List<MesShippingOrderManagementDetail>> collect = shippingDetails.stream().collect(Collectors.groupingBy(MesShippingOrderManagementDetail::getBarcode));
            for (MesWorkOrderPart mesWorkOrderPart : modelList) {
                if (!CollectionUtils.isEmpty(collect.get(mesWorkOrderPart.getWorkOrderNo()))){
                    MesShippingOrderManagementDetail managementDetail = collect.get(mesWorkOrderPart.getWorkOrderNo()).get(0);
                    mesWorkOrderPart.setItemQty(mesWorkOrderPart.getItemQty()*managementDetail.getPlanQty());
                    mesWorkOrderPart.setShippingCode(managementDetail.getShippingCode());
                    mesWorkOrderPart.setTotalPartNo(managementDetail.getPartNo());
                }
            }
        }

        return new ListPager<>(modelList, pager);
    }

}
