package cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.strategy;

import cn.estsh.i3plus.ext.mes.apiservice.serviceimpl.print.IPrintTemplateStrategyService;
import cn.estsh.i3plus.ext.mes.pojo.model.MesProduceSnPrintModel;
import cn.estsh.i3plus.ext.mes.pojo.util.MesExtConstWords;
import cn.estsh.i3plus.platform.common.tool.TimeTool;
import cn.estsh.i3plus.pojo.base.bean.DdlPackBean;
import cn.estsh.i3plus.pojo.base.enumutil.CommonEnumUtil;
import cn.estsh.i3plus.pojo.base.tool.DdlPreparedPack;
import cn.estsh.i3plus.pojo.mes.bean.*;
import cn.estsh.i3plus.pojo.mes.model.GenSerialNoModel;
import cn.estsh.i3plus.pojo.mes.repository.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description : 裁片工单标签打印
 * @Reference :
 * @Author : Castle
 * @CreateDate : 2024/6/17 16:43
 * @Modify:
 **/
@Component
@Slf4j
public class CutWorkOrderNoPrintStrategy implements IPrintTemplateStrategyService {

    @Autowired
    private MesCutSchemeFgRepository cutSchemeFgRepository;

    @Autowired
    private MesCutSchemeMaterialRepository cutSchemeMaterialRepository;

    @Autowired
    private MesCutSchemeEquipmentRepository cutSchemeEquipmentRepository;

    @Autowired
    private MesCutSchemeEquipmentDetailRepository cutSchemeEquipmentDetailRepository;

    public static final int EQUIP_NUM = 1;
    public static final int EQUIP_PARAM_TITLE_NUM = 4;

    @Override
    public MesProduceSnPrintModel execute(GenSerialNoModel genSerialNoModel,
                                          MesProduceSnPrintModel model,
                                          MesNumberRule numberRule) {

        List<MesWorkOrderCut> workOrderCutList = (List<MesWorkOrderCut>)model.getSourceData();
        List<Map<String, Object>> printDataMapList = new ArrayList<>();

        // 裁片方案代码集合
        List<String> cutCodeList = workOrderCutList.stream().map(MesWorkOrderCut::getCutCode).distinct().collect(Collectors.toList());



        Map<String, List<MesCutSchemeFg>> cutSchemeFgMapByCutCode = getCutSchemeFgMapByCutCode(cutCodeList);

        List<MesCutSchemeMaterial> cutSchemeMaterialList = getMesCutSchemeMaterials(cutCodeList);
        //根据裁片方案代码对裁片方案原材料分组
        Map<String, List<MesCutSchemeMaterial>> cutSchemeMaterialMapByCutCode = cutSchemeMaterialList.stream().collect(Collectors.groupingBy(MesCutSchemeMaterial::getCutCode));

        List<MesCutSchemeEquipment> cutSchemeEquipmentList = getMesCutSchemeEquipments(model, cutCodeList);

        cutSchemeEquipmentList = cutSchemeEquipmentList.stream().filter(o -> o.getIsPrint() != null && o.getIsPrint() == CommonEnumUtil.VALID).collect(Collectors.toList());

        // 根据裁片方案代码对裁片设备信息分组
        Map<String, List<MesCutSchemeEquipment>> cutSchemeEquipmentMapByCutCode = CollectionUtils.isEmpty(cutSchemeEquipmentList) ? null : cutSchemeEquipmentList.stream().collect(Collectors.groupingBy(MesCutSchemeEquipment::getCutCode));

        List<MesCutSchemeEquipmentDetail> cutSchemeEquipmentDetailList = getMesCutSchemeEquipmentDetails(model, cutSchemeEquipmentList);

        Map<Long, List<MesCutSchemeEquipmentDetail>> cutSchemeEquipmentDetailMapByPid = CollectionUtils.isEmpty(cutSchemeEquipmentDetailList) ? null : cutSchemeEquipmentDetailList.stream().collect(Collectors.groupingBy(MesCutSchemeEquipmentDetail::getPid));

        for (MesWorkOrderCut workOrderCut : workOrderCutList) {

            List<MesCutSchemeFg> mesCutSchemeFgList = cutSchemeFgMapByCutCode.get(workOrderCut.getCutCode());

            List<MesCutSchemeMaterial> mesCutSchemeMaterialList = cutSchemeMaterialMapByCutCode.get(workOrderCut.getCutCode());

            MesCutSchemeMaterial cutSchemeMaterial = mesCutSchemeMaterialList.get(0);

            List<MesCutSchemeEquipment> cutSchemeEquipmentListByCutCode = CollectionUtils.isEmpty(cutSchemeEquipmentMapByCutCode) ? null : cutSchemeEquipmentMapByCutCode.get(workOrderCut.getCutCode());

            if (!CollectionUtils.isEmpty(cutSchemeEquipmentListByCutCode)) cutSchemeEquipmentListByCutCode.sort(Comparator.comparing(MesCutSchemeEquipment::getSeq));

            Map<String, Object> resultMap = new HashMap<>();

            resultMap.put(MesExtConstWords.CUT_WORK_ORDER_NO, workOrderCut.getCutWorkOrderNo());
            resultMap.put(MesExtConstWords.CUT_NAME, workOrderCut.getCutName());
            // 原材料零件号
            resultMap.put(MesExtConstWords.RAW_PART_NO, cutSchemeMaterial.getPartNo());

            // 拉料信息 START
            resultMap.put(MesExtConstWords.PULLING_MATERIAL_LENGTH, getValByDouble(cutSchemeMaterial.getPullingMaterialLength()));
            resultMap.put(MesExtConstWords.PULLING_MATERIAL_LAYER_NUMBER, getValByDouble(cutSchemeMaterial.getPullingMaterialLayerNumber()));
            resultMap.put(MesExtConstWords.PER_LAYER_NUMBER, getValByDouble(cutSchemeMaterial.getPerLayerNumber()));
            resultMap.put(MesExtConstWords.MATERIAL_DOSAGE, getValByDouble(cutSchemeMaterial.getMaterialDosage()));
            resultMap.put(MesExtConstWords.USING_LENGTH, getValByDouble(cutSchemeMaterial.getUsingLength()));
            resultMap.put(MesExtConstWords.LAYER_NUMBER, getValByDouble(cutSchemeMaterial.getLayerNumber()));
            resultMap.put(MesExtConstWords.TENSION_PARAM, getValByDouble(cutSchemeMaterial.getTensionParam()));
            resultMap.put(MesExtConstWords.PRINT_TIME, TimeTool.getNowTime("yyyy-M-d H:mm:ss"));
            // 拉料信息 END

            MesCutSchemeEquipment mesCutSchemeEquipment = CollectionUtils.isEmpty(cutSchemeEquipmentListByCutCode) ? null : cutSchemeEquipmentListByCutCode.get(0);

            List<MesCutSchemeEquipmentDetail> cutSchemeEquipmentDetails = mesCutSchemeEquipment == null || CollectionUtils.isEmpty(cutSchemeEquipmentDetailMapByPid) ? new ArrayList<>() : cutSchemeEquipmentDetailMapByPid.get(mesCutSchemeEquipment.getId());

            for (int i = 0; i < EQUIP_NUM; i++) {
                //设备参数值标题
                resultMap.put(MesExtConstWords.EQUIPMENT_NAME+(i+1), mesCutSchemeEquipment == null ? "" : mesCutSchemeEquipment.getEquipmentName());
                for (int l = 0; l < EQUIP_PARAM_TITLE_NUM; l++) {
                    if (cutSchemeEquipmentDetails.size() > l) {
                        //设备参数值标题1
                        resultMap.put(MesExtConstWords.EQUIP_PARAM_NAME+(l+1), cutSchemeEquipmentDetails.get(l).getName());
                        //设备参数值1
                        resultMap.put(MesExtConstWords.EQUIP_PARAM_VALUE+(l+1), cutSchemeEquipmentDetails.get(l).getValue());
                    } else {
                        //设备参数值标题1
                        resultMap.put(MesExtConstWords.EQUIP_PARAM_NAME+(l+1), "");
                        //设备参数值1
                        resultMap.put(MesExtConstWords.EQUIP_PARAM_VALUE+(l+1), "");
                    }
                }
            }

            String partValue = "";
            if (!CollectionUtils.isEmpty(mesCutSchemeFgList)) {
                BigDecimal[] bigDecimals = divideAndRemainder(mesCutSchemeFgList.get(0).getQty(), mesCutSchemeFgList.get(0).getPackageQty());
                BigDecimal remainder = bigDecimals[1];
                Double boxQty = remainder.doubleValue() > 0 ? bigDecimals[0].add(new BigDecimal("1")).doubleValue() : bigDecimals[0].doubleValue();
                partValue = mesCutSchemeFgList.get(0).getPartNo() + "    " + boxQty.intValue() +"箱";
            }

            //产品信息
            resultMap.put(MesExtConstWords.FG_PART_NO, partValue);

            printDataMapList.add(resultMap);

        }

        model.setPrintContextList(packResultMapList(model, printDataMapList));

        return model;
    }

    private String getValByDouble(Double val) {
        return val == null ? "" : val.toString();
    }

    private Map<String, List<MesCutSchemeFg>> getCutSchemeFgMapByCutCode(List<String> cutCodeList) {
        List<MesCutSchemeFg> cutSchemeFgList = getMesCutSchemeFgs(cutCodeList);

        //根据裁片方案代码对裁片方案成品明细分组
        Map<String, List<MesCutSchemeFg>> cutSchemeFgMapByCutCode = cutSchemeFgList.stream().collect(Collectors.groupingBy(MesCutSchemeFg::getCutCode));
        return cutSchemeFgMapByCutCode;
    }

    private BigDecimal[] divideAndRemainder(Double qty, Double packageQty) {
        BigDecimal dividend  = new BigDecimal(qty.toString());
        BigDecimal divisor = new BigDecimal(packageQty.toString());
        return dividend.divideAndRemainder(divisor);
    }

    private List<MesCutSchemeEquipmentDetail> getMesCutSchemeEquipmentDetails(MesProduceSnPrintModel model, List<MesCutSchemeEquipment> cutSchemeEquipmentList) {
        if (!CollectionUtils.isEmpty(cutSchemeEquipmentList)) {
            List<Long> idList = cutSchemeEquipmentList.stream().map(MesCutSchemeEquipment::getId).collect(Collectors.toList());
            DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
            DdlPreparedPack.getInPackList(idList, MesExtConstWords.PID, packBean);
            return cutSchemeEquipmentDetailRepository.findByHqlWhere(packBean);
        }
        return null;
    }

    private List<MesCutSchemeEquipment> getMesCutSchemeEquipments(MesProduceSnPrintModel model, List<String> cutCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean(model.getOrganizeCode());
        DdlPreparedPack.getInPackList(cutCodeList, MesExtConstWords.CUT_CODE, packBean);
        List<MesCutSchemeEquipment> cutSchemeEquipmentList = cutSchemeEquipmentRepository.findByHqlWhere(packBean);
        return cutSchemeEquipmentList;
    }

    private List<MesCutSchemeMaterial> getMesCutSchemeMaterials(List<String> cutCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(cutCodeList, MesExtConstWords.CUT_CODE, packBean);
        List<MesCutSchemeMaterial> cutSchemeMaterialList = cutSchemeMaterialRepository.findByHqlWhere(packBean);
        return cutSchemeMaterialList;
    }

    private List<MesCutSchemeFg> getMesCutSchemeFgs(List<String> cutCodeList) {
        DdlPackBean packBean = DdlPackBean.getDdlPackBean();
        DdlPreparedPack.getInPackList(cutCodeList, MesExtConstWords.CUT_CODE, packBean);
        List<MesCutSchemeFg> cutSchemeFgList = cutSchemeFgRepository.findByHqlWhere(packBean);
        return cutSchemeFgList;
    }

    private List<Map<String, Object>> packResultMapList(MesProduceSnPrintModel printModel, List<Map<String, Object>> printTemplateDateList) {
        List<Map<String, Object>> resultMapList = new ArrayList<>();
        Map<String, Object> resultMap = new HashMap<>();
        resultMap.put(MesExtConstWords.LABEL_TEMPLATE, printModel.getMesLabelTemplate());
        resultMap.put(MesExtConstWords.TEMPLATE_DATA, printTemplateDateList);
        resultMap.put(MesExtConstWords.TEMPLATE_CODE, printModel.getMesLabelTemplate().getTemplateCode());
        resultMap.put(MesExtConstWords.PRINTER, printModel.getPrinter());
        resultMapList.add(resultMap);
        return resultMapList;
    }

}
